/*
	Paintable Computing Simulation

	License

	This software (Paintable Computing Simulation and all parts of its 
	source code) is provided "as is", without any warranty and it is 
	the property of its respective authors. Use of this software is 
	free to everyone, except in commercial and/or military purposes 
	without written permission of its authors. Disassembling the 
	executable code of this software is forbidden for any purpose 
	without written permission of its authors.

	If you intend to use any part of the source code of this software 
	or to develop this software any further, you must include this 
	license in every source file and every About box, without changing 
	it; only appending the license is allowed. And if you use any part 
	of the source code of this software, or develop it further, please 
	send an e-mail to its authors.

	Authors of this software are not responsible for any consequences 
	which use of this software, use of any part of its source or its 
	further development may cause.

	All software and parts of source code used in the development of 
	this software are property of their respective owners.

	Good luck,
	Authors

	Authors (alphabetically):
	Fabijan Lukin
		fabijan@gmail.com
		fabijan.lukin@fer.hr
	Fran Pregernik
		fran.pregernik@gmail.com
		fran.pregernik@fer.hr
	Tomislav Sukser
		tomislav.sukser@gmail.com
		tomislav.sukser@zg.t-com.hr
		tomislav.sukser@fer.hr


	Note:
	Paintable computing was firstly introduced in Programming a 
	Paintable Computer written by William Joseph Butera.

    Copyright (c) 2005,2006. All rights reserved.
    4-6/2006

 */
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Globalization;
using System.IO;
using System.Reflection;
using System.Threading;
using System.Windows.Forms;

using AviLib;
using PaintableSimulator.Engine;
using PaintableSimulator.Engine.PaintableComputers.OperatingSystem;
using PaintableSimulator.Engine.PaintableComputers.Sensors;
using PaintableSimulator.Visualisation;

namespace PaintableSimulator.GUI.Standard
{
	/// <summary>
	/// Windows Forma za prikaz simulatora
	/// </summary>
	public partial class frmGUI : Form
	{
		/// <summary>
		/// 
		/// </summary>
		private enum DisabledGroups
		{
			ProgramStart,
			SimulationStarted
		}

		/// <summary>
		/// 
		/// </summary>
		private enum MouseClick
		{
			Nothing,
			Infect,
			InfectSelect,
			ShowInfo,
            InfoShowed,
			TransparentRegion,
			TransparentRegionDone,
            SenzorAdd
		} ;

		/// <summary>
		/// What will happend when user clicks on display panel
		/// </summary>
		private MouseClick MouseClickDoes = MouseClick.Nothing;

		/// <summary>
		/// Simulator object
		/// </summary>
		private GenericSimulator simulator = null;

		/// <summary>
		/// 
		/// </summary>
		private Visualize display = null;
		private AVIExport AviFile = null;


        List<Type> SensorTypes = new List<Type>();
        List<Point> SensorLocations = new List<Point>();

		/// <summary>
		/// 
		/// </summary>
		private FriendlyType internalColorizer = null;

		#region Infothread

		private InfoThreadSychronization infoThreadSynchro;

		private void infoThreadFunction()
		{
			infoThreadSynchro.DoWork = true;
			InfoThread thread = new InfoThread(this, ref infoThreadSynchro);
			thread.Run();
		}

		/// <summary>
		/// 
		/// </summary>
		public delegate void ShowInfo();

		/// <summary>
		/// 
		/// </summary>
		public ShowInfo doShowInfo;

		private bool internalForceDraw = false;


        private void DrawAndPaint()
        {
            List<Color> colors = null;
            // preprevio sam castanje u FriendlyType tako da ne treba vise onaj if
            colors = simulator.ColorizeHomepages(internalColorizer);

            // PC
            display.DoDraw(simulator.CustomComputerList, colors, internalForceDraw);
            if (internalForceDraw)
                internalForceDraw = false;
            // pa na ekran
            display.DoPaint();
        }
		/// <summary>
		/// 
		/// </summary>
		public void ShowInfoFunction()
		{
			if ((display != null) && (mnuDoShowScreen.Checked))
			{
				if (AviFile == null)
				{
                    DrawAndPaint();
                }
                else
                {
                    //4 framea in a row
                    simulator.Stop();
                    DrawAndPaint();
					AviFile.SaveFrame(display.ReturnBitmap());

                    simulator.Step();
                    DrawAndPaint();
                    AviFile.SaveFrame(display.ReturnBitmap());

                    simulator.Step();
                    DrawAndPaint();
                    AviFile.SaveFrame(display.ReturnBitmap());

                    simulator.Step();
                    DrawAndPaint();
                    AviFile.SaveFrame(display.ReturnBitmap());
                    //
                    simulator.Run();
                }

				// Ako je ovo jednokratno prikazivanje
				if (mnuShowCurrentScreen.Checked)
				{
					mnuDoShowScreen.Checked = false;
					mnuShowCurrentScreen.Checked = false;
				}
			}

			// Da ne treperi, ovo osvjezavamo svake sekunde...
			if (FPS.SecondGone())
			{   
                
                if (MouseClickDoes ==  MouseClick.InfoShowed)
                    if (display != null)
                        DisplayPCInfo(display.GetInfoPC());


				// Log
				LogEntry log = simulator.ReadLog();
				while (log != null)
				{
					listboxDebug.Items.Add(log.TimeStamp.ToLongTimeString() + "  -  " + log.Message);
					listboxDebug.SelectedIndex = listboxDebug.Items.Count - 1;
					log = simulator.ReadLog();
				}

				// Info
				SimulatorInfo simulatorInfo = simulator.SimulatorStatus;
				listboxInfo.Items[0].SubItems[1].Text = simulatorInfo.TickCount.ToString() +
				                                        " (" + FPS.GetFPS(simulatorInfo.TickCount) + " ips)";

				listboxInfo.Items[1].SubItems[1].Text = simulatorInfo.Communication.PendingPacketCount.ToString();
				uint tmpOne = simulatorInfo.Communication.LostPacketCount;
				UInt64 tmpTwo = simulatorInfo.Communication.TotalPacketCount;
				listboxInfo.Items[2].SubItems[1].Text = tmpOne.ToString() + " / " +
				                                        tmpTwo.ToString() + " (" +
				                                        ((tmpOne*100F)/tmpTwo).ToString() + ")";
			}
		}

		#endregion Infothread

		#region Pomocne funkcije

        double GetSelectedZoomFactor()
        {
            // Zoom
            foreach (ToolStripMenuItem tsmi in zoomToolStripMenuItem.DropDownItems)
            {
                if (tsmi.Checked)
                {
                    return Convert.ToInt32(tsmi.Tag) / 10.0;
                    break;
                }
            }
            return 1.0;
        }

		/// <summary>
		/// Brie tabove iz kontrole, te skriva kontrolu ako je jedina
		/// </summary>
		/// <param name="tp"></param>
		public void DoRemoveTab(TabPage tp)
		{
			tabOpcije.TabPages.Remove(tp);
            if (tabOpcije.TabCount <= 0)
            {
                tabOpcije.Hide();
                hSplit.Hide();
            }
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="tp"></param>
		public void DoAddTab(TabPage tp)
		{
			tabOpcije.TabPages.Add(tp);
			if (tabOpcije.TabCount > 0)
				tabOpcije.Show();
			tp.Show();
            hSplit.Show();
		}

		/// <summary>
		/// Finds out available display modes
		/// </summary>
		public void ScanDisplays()
		{
			// Pronadjimo sve clasove unutar ProcessFragmentExamples
			Assembly objAssembly = Assembly.GetAssembly(typeof (Display));
			// now we loaded the assembly
			int LastIndex = mnuGraphicDisplay.DropDownItems.IndexOf(mnuDisplaySeparator) + 1;
			if (objAssembly != null)
			{
				// dodajmo to u listu
				while (mnuGraphicDisplay.DropDownItems.Count > LastIndex)
					mnuGraphicDisplay.DropDownItems.RemoveAt(LastIndex);

				Type[] types = objAssembly.GetTypes();

				KeysConverter shortCuts = new KeysConverter();
				KeysConverter k = new KeysConverter();
				for (int i = 0; i < types.Length; i++)
				{
					try
					{
						if (types[i].BaseType == typeof (Display))
						{
							ToolStripMenuItem tsmi = new ToolStripMenuItem(types[i].ToString(), null, mnuGraphicDisplay_Click);
							tsmi.Tag = types[i];
							if (mnuGraphicDisplay.DropDownItems.Count < 10)
								tsmi.ShortcutKeys =
									(Keys) k.ConvertFromInvariantString("Ctrl+" + mnuGraphicDisplay.DropDownItems.Count.ToString());
							else if (mnuGraphicDisplay.DropDownItems.Count < 20)
								tsmi.ShortcutKeys =
									(Keys) k.ConvertFromInvariantString("Ctrl+Alt+" + (mnuGraphicDisplay.DropDownItems.Count - 10).ToString());

							mnuGraphicDisplay.DropDownItems.Add(tsmi);
						}
					}
					catch
					{
						continue;
					}
				}

				// Izaberemo 1.
				if (mnuGraphicDisplay.DropDownItems.Count > LastIndex)
					mnuGraphicDisplay.DropDownItems[LastIndex].PerformClick();
			}
			else
				throw new Exception(""); //TODO: create custom exception
		}


        /// <summary>
        /// Reads the ProcessFragment subdirectory for ProcessFragment dll's into a tree view
        /// </summary>
        private void ScanProcesFragments(TreeView tw)
        {

            // clear the treeview
            tw.Nodes.Clear();

			List<FriendlyType> fragments = simulator.Plugins.ValidProcessFragments;

            // load assemblies
            for (int i = 0; i < fragments.Count; i++)
            {
                string assemblyName = fragments[i].InnerType.Assembly.GetName().Name;

                TreeNode assemblyNode = null;
				if (tw.Nodes.ContainsKey(fragments[i].InnerType.Assembly.FullName))
					assemblyNode = tw.Nodes[fragments[i].InnerType.Assembly.FullName];
                else
					assemblyNode = tw.Nodes.Add(fragments[i].InnerType.Assembly.FullName, assemblyName);

                // make an entry for each
                TreeNode typeNode = new TreeNode(fragments[i].UserDefinedName);

                // The tag contains the type that is used to instantiate the pfrag
                typeNode.Tag = fragments[i];

                // set tool tip
				typeNode.ToolTipText = fragments[i].UserDefinedDescription;

                // add it to the assembly node
                assemblyNode.Nodes.Add(typeNode);

            }


            // expand all nodes
            tw.ExpandAll();
        }
        
		#endregion Pomocne funkcije

		/// <summary>
		/// Selects display mode
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		/// <summary>
		/// Form Entry Point
		/// </summary>
		public frmGUI()
		{
			Thread.CurrentThread.CurrentCulture = new CultureInfo("hr-HR");
			Thread.CurrentThread.CurrentUICulture = new CultureInfo("hr-HR");

			InitializeComponent();

			// Za sada samo GDI prikaz
			SetStyle(ControlStyles.UserPaint, true);
			SetStyle(ControlStyles.AllPaintingInWmPaint, true);
			SetStyle(ControlStyles.OptimizedDoubleBuffer, true);

			// Kako sve znamo crtati
			ScanDisplays();

			// Makni sve tabove
			while (tabOpcije.TabPages.Count > 0)
			{
				DoRemoveTab(tabOpcije.TabPages[0]);
			}

			// HACK
			// Ponekad mi se glavni meni ne prikazuje?!?!?
			menuMain.Visible = true;

			// Neka je po defaultu omogucena komunikacija
			mnuCommunication.PerformClick();

			//Delegate koji poziva InfoThread i u kojemu se
			// prikazuje info i ekran
			doShowInfo = new ShowInfo(ShowInfoFunction);

			// 
			EnableGroup(DisabledGroups.ProgramStart);
		}

		/// <summary>
		/// 
		/// </summary>
		private void Terminate()
		{
			// "Zaboravimo" sve tipove
            internalColorizer = null;
            for (int i = 1; (i < mnuViewTypes.DropDownItems.Count) && (mnuViewTypes.DropDownItems.Count > 2); i++)
			{
                if (mnuViewTypes.DropDownItems[i] != mnuSenzorSeparator)
                {
        			mnuViewTypes.DropDownItems.RemoveAt(i);
                    i--;
                }
			}
			// AVI?
			if (AviFile != null)
			{
				AviFile.CloseAviFile();
				AviFile = null;
			}


			if (simulator != null)
			{
				// Ubijamo info thread
				infoThreadSynchro.DoWork = false;
				Thread.Sleep(1000);

				//
				if (display != null)
				{
					display.Terminate();
					display = null;
				}

				// Ubijamo simulator
				simulator.Terminate();
				simulator = null;
			}
		}

		#region Enable and Disable Components

		/// <summary>
		/// Disables all unwanted components for a selected group
		/// Groups are explaned below
		/// </summary>
		/// <param name="groupName"></param>
		private void EnableGroup(DisabledGroups groupName)
		{
			//
			menuMain.Enabled = true;
			toolStrip.Enabled = true;

			switch (groupName)
			{
					//
				case DisabledGroups.SimulationStarted:

					mnuNewSimulation.Enabled = btnNewSimulation.Enabled = false;
                    mnuNewDefaultSimulation.Enabled = mnuNewSimulation.Enabled;
                    mnuCancelSimulation.Enabled = btnCancelSimulation.Enabled = (tabOpcije.TabCount == 0);
					mnuStartStop.Enabled = btnStartStop.Enabled = true;
					mnuStep.Enabled = btnStep.Enabled = (simulator.SimulatorStatus.SimulatorState == SimulatorStates.Stopped);
					mnuInvadePC.Enabled = btnInvadePC.Enabled = (tabOpcije.TabCount == 0);
                    mnuInsertSenzor.Enabled = btnInsertSenzor.Enabled = (tabOpcije.TabCount == 0);

					mnuShowPCInfo.Enabled = btnShowPCInfo.Enabled = ((tabOpcije.TabCount == 0) || (mnuShowPCInfo.Checked));
					mnuCommunication.Enabled = btnCommunication.Enabled = true;

					mnuDeleteRegion.Enabled = (tabOpcije.TabCount == 0);
    				mnuShowCurrentScreen.Enabled = true;

					break;

					//
				case DisabledGroups.ProgramStart:
				default:
					//
					mnuNewSimulation.Enabled = btnNewSimulation.Enabled = true;
                    mnuNewDefaultSimulation.Enabled = mnuNewSimulation.Enabled;
					mnuCancelSimulation.Enabled = btnCancelSimulation.Enabled = false;
					mnuStartStop.Enabled = btnStartStop.Enabled = false;
					mnuStep.Enabled = btnStep.Enabled = false;
                    mnuInsertSenzor.Enabled = btnInsertSenzor.Enabled = false;
					mnuInvadePC.Enabled = btnInvadePC.Enabled = false;
					mnuShowPCInfo.Enabled = btnShowPCInfo.Enabled = false;
					mnuCommunication.Enabled = btnCommunication.Enabled = false;


					mnuDeleteRegion.Enabled = false;
					mnuShowCurrentScreen.Enabled = false;

					break;
			}
		}

		#endregion Enable and Disable Components

		#region OnClick Events

		private void mnuAbout_Click(object sender, EventArgs e)
		{
			frmAbout about = new frmAbout();
			about.ShowDialog(this);
		}

        private void CalculateScrollbars()
        {
            display.ZoomFactor = GetSelectedZoomFactor();
            panDisplay.Width = display.DisplaySize.Width;
            panDisplay.Height = display.DisplaySize.Height;
            display.RefreshDisplaySize();
        }
        private void SetupDisplay()
        {
            if (display == null) return;

            display.OriginPoint = new Point(0, 0);
            display.SetDisplayWindow((Type)mnuGraphicDisplay.Tag,
                                     this.panDisplay.Handle,
                                     simulator.SimulatorStatus.Environment.WallSize);
            display.PaintableComputerDisplaySize = 3;
            display.WirelessRadius = simulator.SimulationSettings.WirelessRadius;

            CalculateScrollbars();

        }
		private void mnuGraphicDisplay_Click(object sender, EventArgs e)
		{
			//

			//if ((sender as ToolStripMenuItem).Tag != null)
			if ((sender as ToolStripMenuItem) != mnuGraphicDisplay)
			{
				for (int i = mnuGraphicDisplay.DropDownItems.IndexOf(mnuDisplaySeparator) + 1;
				     i < mnuGraphicDisplay.DropDownItems.Count;
				     i++)
					(mnuGraphicDisplay.DropDownItems[i] as ToolStripMenuItem).Checked = false;

				(sender as ToolStripMenuItem).Checked = true;
				mnuGraphicDisplay.Tag = (sender as ToolStripMenuItem).Tag;

                SetupDisplay();
			}
		}

		private void mnuNewSimulation_Click(object sender, EventArgs e)
		{
            // Stvaramo simulator
            simulator = new GenericSimulator();
            simulator.CommunicationEnabled = mnuCommunication.Checked;

            // Koje programe znamo izvoditi
            ScanProcesFragments(processFragments);

            //Defaultni timovi
            // da se mogu senzori izabrati
            cbDefaultTypes.Items.Clear();
            FriendlyType[] types = simulator.Plugins.ValidPaintableComputers.ToArray();
            cbDefaultTypes.Items.AddRange(types);
            cbDefaultTypes.SelectedIndex = 0;

            
            //
			btnNewSimulation.Enabled = false;
			mnuNewSimulation.Enabled = false;
            mnuNewDefaultSimulation.Enabled = false;

			propertyEngineSettings.SelectedObject = simulator.SimulationSettings;
			txtWirelesRadius_ValueChanged(null, EventArgs.Empty);

            if (sender != null)
                DoAddTab(tabNovaSimulacija);
		}

		private void btnDoCancelSimulation_Click(object sender, EventArgs e)
		{
            if (simulator != null)
            {
                if (simulator.SimulatorStatus.SimulatorState == SimulatorStates.Running)
                    simulator.Terminate();
                simulator = null;
            }
			DoRemoveTab(tabNovaSimulacija);
			EnableGroup(DisabledGroups.ProgramStart);
		}

		private void btnDoStartSimulation_Click(object sender, EventArgs e)
		{
			DoRemoveTab(tabNovaSimulacija);

			//
            //simulator.InitializeSimulator(typeof(PaintableSimulator.Engine.PaintableComputers.TemperatureSensorPaintableComputer));
            simulator.InitializeSimulator((FriendlyType) cbDefaultTypes.SelectedItem);
            if (((FriendlyType)cbDefaultTypes.SelectedItem).IsSensor)
                AddColorizerToMenu((FriendlyType)cbDefaultTypes.SelectedItem);

			//Display
			display = new Visualize();
            SetupDisplay();
            display.ZoomFactor = GetSelectedZoomFactor();
            display.BoldSensorsAndPortals = mnuBoldSensorsAndPortals.Checked;

			//
			FPS.LastUpdateTime = DateTime.Now;
			infoThreadSynchro = new InfoThreadSychronization();
			Thread tmpThread = new Thread(new ThreadStart(infoThreadFunction));
			tmpThread.Start();


			EnableGroup(DisabledGroups.SimulationStarted);

			mnuStartStop.PerformClick();

			AviFile = null;
			if (txtAVIOut.Text.Trim() != "")
			{
				AviFile = new AVIExport();
				switch (Convert.ToInt32(mnuAviExport.Tag))
				{
                    case 2:
                        AviFile.CreateAviFile(txtAVIOut.Text.Trim(), AviCompression.HuffYUV);
                        break;
                    
                    case 1:
						AviFile.CreateAviFile(txtAVIOut.Text.Trim(), AviCompression.Indeo5);
						break;

					case 0:
					default:
						AviFile.CreateAviFile(txtAVIOut.Text.Trim(), AviCompression.None);
						break;
				}
			}
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void mnuCancelSimulation_Click(object sender, EventArgs e)
		{
			//ToDo: Zapravo, trebalo bi zabraniti zatvaranje prozora na onaj X, i forsirati
			// zatvaranje preko izbornika!
			if (mnuShowPCInfo.Checked)
				mnuShowPCInfo.PerformClick();

            if (MessageBox.Show("Da li stvarno elite prekinuti simulaciju?", "Paintable Computer Simulator.GUI",
                                MessageBoxButtons.YesNo) == DialogResult.Yes)
            {
                Terminate();
                EnableGroup(DisabledGroups.ProgramStart);
            }
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void mnuStartStop_Click(object sender, EventArgs e)
		{
			if (simulator.SimulatorStatus.SimulatorState == SimulatorStates.Running)
			{
				simulator.Stop();
			}
			else if ((simulator.SimulatorStatus.SimulatorState == SimulatorStates.Stopped) ||
			         (simulator.SimulatorStatus.SimulatorState == SimulatorStates.ReadyToRun))
			{
				simulator.Run();
			}
			EnableGroup(DisabledGroups.SimulationStarted);
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void mnuStep_Click(object sender, EventArgs e)
		{
			if ((simulator.SimulatorStatus.SimulatorState == SimulatorStates.Running) ||
			    (simulator.SimulatorStatus.SimulatorState == SimulatorStates.Stopped))
			{
				simulator.Step();
			}
		}


		/// <summary>
		/// 
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void frmGUI_FormClosing(object sender, FormClosingEventArgs e)
		{
            e.Cancel = (simulator != null);
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void mnuInvadePC_Click(object sender, EventArgs e)
		{
			MouseClickDoes = MouseClick.Infect;

			txtFragDescription.Text = "";
			txtInvadeThisPC.Tag = null;
			txtInvadeThisPC.Text = "";

			btnDisplayOptions.Text = "Opcije";
			btnDisplayOptions.Tag = 0;
			btnDisplayOptions.Enabled = false;

			processFragments.SelectedNode = null;

			while (tabPFragSelect.TabPages.Count > 0)
			{
				tabPFragSelect.TabPages.RemoveAt(0);
			}
			tabPFragSelect.TabPages.Add(tabPFragSelection);
			DoAddTab(tabPFrags);

			EnableGroup(DisabledGroups.SimulationStarted);
		}

		private void mnuShowPCInfo_Click(object sender, EventArgs e)
		{
			mnuShowPCInfo.Checked = btnShowPCInfo.Checked = !(mnuShowPCInfo.Checked);

			if (mnuShowPCInfo.Checked)
			{
				lwPCInfo.Items.Clear();
				//
				ListViewItem item;

				//////////
				// Osnovne informacije
				ListViewGroup lwg = lwPCInfo.Groups[0];
				//
				item = new ListViewItem("ID", lwg);
				item.Name = "ID";
				item.SubItems.Add("0");
				lwPCInfo.Items.Add(item);

				//
				item = new ListViewItem("Lokacija", lwg);
				item.Name = "Lokacija";
				item.SubItems.Add("(X:Y) = (?:?)");
				lwPCInfo.Items.Add(item);

				//
				item = new ListViewItem("IsAlive", lwg);
				item.Name = "IsAlive";
				item.SubItems.Add("?");
				lwPCInfo.Items.Add(item);


				//////////
				// Homepages
				lwg = lwPCInfo.Groups[1];
				item = new ListViewItem("Homepage Count", lwg);
				item.Name = "HomepageCount";
				item.SubItems.Add("?");
				lwPCInfo.Items.Add(item);

				//
				DoAddTab(tabDebugPC);
				panDisplay.Cursor = Cursors.NoMove2D;
				MouseClickDoes = MouseClick.ShowInfo;
			}
			else
			{
				DoRemoveTab(tabDebugPC);
				panDisplay.Cursor = Cursors.Default;
				MouseClickDoes = MouseClick.Nothing;
                if (display != null)
                    display.SetInfoPC(null);
			}
			EnableGroup(DisabledGroups.SimulationStarted);
		}


        private void FillNodes(TreeNode parent, Homepage hp)
        {
            if (hp != null)
            {
                for (int i = 0; i < hp.KeyList.Count; i++)
                {
                    string text = "ProcessFragment = {0}";
                    TreeNode tnKey = new TreeNode(String.Format(text, hp.KeyList[i].ToString("X")));
                    for (int j = 0; j < hp[hp.KeyList[i]].Count; j++)
                    {
                        string post = "POST_{0} = {1}";
                        string label;

                        if (hp[hp.KeyList[i]][j] != null)
                            label = String.Format(post, j, hp[hp.KeyList[i]][j].ToString());
                        else
                            label = String.Format(post, j, "null");

                        TreeNode tnValue = new TreeNode(label);

                        // loop through process fragment post array list if possible
                        if ((hp[hp.KeyList[i]][j] != null) && (hp[hp.KeyList[i]][j] is Array))
                        {
                            Array array = (Array)hp[hp.KeyList[i]][j];
                            for (int k = 0; k < array.Length; k++)
                            {
                                string arrayLabel = String.Format("Item[{0}] = {1}", k, array.GetValue(k).ToString());
                                tnValue.Nodes.Add(new TreeNode(array.GetValue(k).ToString()));

                            }
                        }
                        tnKey.Nodes.Add(tnValue);
                    }
                    parent.Nodes.Add(tnKey);
                }
            }
        }

        private bool MustreRedraw(TreeNode node1, TreeNode node2)
        {
            bool check = true;
            if (node1.GetNodeCount(false) == node2.GetNodeCount(false))
            {
                for (int i = 0; i < node1.GetNodeCount(false); i++)
                {
                    if (node1.Nodes[i].Text != node2.Nodes[i].Text)
                    {
                        check = false;
                        break;
                    }
                    if (!MustreRedraw(node1.Nodes[i], node2.Nodes[i]))
                    {
                        check = false;
                        break;
                    }
                }
            }
            else
            {
                check = false;
            }
            return check;

        }

        private void DisplayPCInfo(PConTheWall tempPC)
        {
            if (tempPC != null)
            {                
                //
                display.SetInfoPC(tempPC);

                ListViewGroup lwg = lwPCInfo.Groups[0];
                lwPCInfo.Items["ID"].SubItems[1].Text = Convert.ToString(tempPC.PaintableComputer.ComputerId);
                lwPCInfo.Items["Lokacija"].SubItems[1].Text = "(X:Y) = (" + tempPC.Location.X.ToString() + ":" +
                                                              tempPC.Location.Y.ToString() + ")";
                lwPCInfo.Items["IsAlive"].SubItems[1].Text = tempPC.IsAlive.ToString();

                // GENERATED BY ITSELF
                TreeView tv = new TreeView();
                tv.Visible = false;

                //
                TreeNode parentNode = new TreeNode(tempPC.PaintableComputer.ComputerId.ToString());
                FillNodes(parentNode, tempPC.PaintableComputer.LocalHomepage);
                tv.Nodes.Add(parentNode);
                // END OF GENERATED BY ITSELF

                // redraw pfraginfo only if necessary ;)
                if ((tv.Nodes.Count != twPFragHomepages.Nodes.Count) || 
                    (!MustreRedraw(tv.Nodes[0], twPFragHomepages.Nodes[0])))
                {
                    twPFragHomepages.BeginUpdate();

                    twPFragHomepages.Nodes.Clear();
                    //
                    parentNode = new TreeNode(tempPC.PaintableComputer.ComputerId.ToString());
                    FillNodes(parentNode, tempPC.PaintableComputer.LocalHomepage);
                    twPFragHomepages.Nodes.Add(parentNode);
                    twPFragHomepages.ExpandAll();

                    twPFragHomepages.EndUpdate();
                }
                
                //
                tv = new TreeView();
                tv.Visible = false;

                TreeNode parentNode2;
                parentNode = new TreeNode(tempPC.PaintableComputer.ComputerId.ToString());
                List<Homepage> mirrors = tempPC.PaintableComputer.MirroredHomepages;
				for (int m=0; m<mirrors.Count; m++)
                {
                    parentNode2 = new TreeNode("Mirror " + m.ToString());
                    FillNodes(parentNode2, mirrors[m]);
                    parentNode.Nodes.Add(parentNode2);
                }
                tv.Nodes.Add(parentNode);

                // redraw pfraginfo only if necessary ;)
                if ((tv.Nodes.Count != twPFragMirroredHomepages.Nodes.Count) ||
                    (!MustreRedraw(tv.Nodes[0], twPFragMirroredHomepages.Nodes[0])))
                 {
                    twPFragMirroredHomepages.BeginUpdate();

                    twPFragMirroredHomepages.Nodes.Clear();
                    //
                    parentNode = new TreeNode(tempPC.PaintableComputer.ComputerId.ToString());
					List<Homepage> mirrors2 = tempPC.PaintableComputer.MirroredHomepages;
					for (int m = 0; m < mirrors2.Count; m++)
					{
						parentNode2 = new TreeNode("Mirror " + m.ToString());
                        FillNodes(parentNode2, mirrors2[m]);
                        parentNode.Nodes.Add(parentNode2);
                    }
                    twPFragMirroredHomepages.Nodes.Add(parentNode);
                    twPFragMirroredHomepages.ExpandAll();

                    twPFragMirroredHomepages.EndUpdate();
                }
            }
            else
            {
                //
                display.SetInfoPC(null);

                lwPCInfo.Items["ID"].SubItems[1].Text = "?";
                lwPCInfo.Items["Lokacija"].SubItems[1].Text = "(X:Y) = (?:?)";
                lwPCInfo.Items["IsAlive"].SubItems[1].Text = "?";

                twPFragHomepages.Nodes.Clear();
                twPFragMirroredHomepages.Nodes.Clear();
            }

        }
		/// <summary>
		/// 
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void panDisplay_MouseMove(object sender, MouseEventArgs e)
		{
			if (simulator == null) return;

			//
			switch (MouseClickDoes)
			{
				case MouseClick.ShowInfo:
					PConTheWall tempPC =
						simulator.GetComputerInfoFromPoint(display.FixCoordinates(new Point(e.X, e.Y)));

                    DisplayPCInfo(tempPC);
					break;

					//
				case MouseClick.TransparentRegion:

					if (display != null)
					{
						// ToDo: Fix GDI glitch
						display.AddTransparentPoint(display.FixCoordinates(new Point(e.X, e.Y)), true);
					}
					break;
			}
		}

		private void panDisplay_MouseDown(object sender, MouseEventArgs e)
		{
			if (simulator == null) return;

            PConTheWall pc;
			switch (MouseClickDoes)
			{
					//
				case MouseClick.ShowInfo:

                    MouseClickDoes = MouseClick.InfoShowed;
                    panDisplay.Cursor = Cursors.Default;
					break;

                case MouseClick.InfoShowed:

                    mnuShowPCInfo.PerformClick();
                    break;


                case MouseClick.SenzorAdd:
                    pc = simulator.GetComputerInfoFromPoint(display.FixCoordinates(new Point(e.X, e.Y)));
                    // Ne smijemo senzor staviti na neki postojeci PC
                    if (pc == null)
                    {
                        ListViewItem lvi = lwSenzersToAdd.Items.Add(twSenzors.SelectedNode.Text);
                        lvi.SubItems.Add(display.FixCoordinates(new Point(e.X, e.Y)).ToString());

                        SensorTypes.Add(twSenzors.SelectedNode.Tag.GetType() );
                        SensorLocations.Add(display.FixCoordinates(new Point(e.X, e.Y)));

                    }
                    btnInsertSensors.Enabled = (lwSenzersToAdd.Items.Count > 0);

                    break;

					//
				case MouseClick.InfectSelect:

					txtInvadeThisPC.Text = "";

					pc = simulator.GetComputerInfoFromPoint(display.FixCoordinates(new Point(e.X, e.Y)));
					if (pc != null)
					{
						int index = simulator.CustomComputerList.IndexOf(pc);
						txtInvadeThisPC.Tag = index;
						txtInvadeThisPC.Text = pc.PaintableComputer.ComputerId.ToString();
					}
					break;

					//
				case MouseClick.TransparentRegion:
					if (e.Button == MouseButtons.Left)
					{
						if (display != null)
						{
							display.AddTransparentPoint(display.FixCoordinates(new Point(e.X, e.Y)), false);
						}
						btnMassKillExecute.Enabled = false;
					}
					else if (e.Button == MouseButtons.Right)
					{
						if (display != null)
						{
							if (display.GetTransparentPoints().Length > 2)
							{
								btnMassKillExecute.Enabled = true;
								MouseClickDoes = MouseClick.TransparentRegionDone;
							}
                            else
                                btnMassKillExecute.Enabled = Convert.ToDouble(numDisableRandomPercentage.Value) > 0;
							panDisplay.Cursor = Cursors.Default;
						}
					}
					break;
			}
		}

		private void mnuCommunication_Click(object sender, EventArgs e)
		{
			btnCommunication.Checked = mnuCommunication.Checked = !mnuCommunication.Checked;

			if (simulator == null) return;
			simulator.CommunicationEnabled = mnuCommunication.Checked;
		}

		private void btnInvadeCancel_Click(object sender, EventArgs e)
		{
			DoRemoveTab(tabPFrags);
			MouseClickDoes = MouseClick.Nothing;
			processFragments.SelectedNode = null;

			EnableGroup(DisabledGroups.SimulationStarted);
		}

        /// <summary>
        /// 
        /// </summary>
        /// <param name="colorizer"></param>
        /// <param name="isSensor"></param>
        private void AddColorizerToMenu(FriendlyType colorizer)
        {
            // Sredimo listu onoga sto se prikazuje...
            bool Add = true;
           
            for (int i = 0; i < mnuViewTypes.DropDownItems.Count; i++)
            {
                if (mnuViewTypes.DropDownItems[i].Tag == null) continue;
                // To String radi "friendly" tipova...
                if (mnuViewTypes.DropDownItems[i].Tag.ToString() == colorizer.ToString())
                {
                    if (!((ToolStripMenuItem)mnuViewTypes.DropDownItems[i]).Checked)
                        mnuViewTypes.DropDownItems[i].PerformClick();
                    Add = false;
                    break;
                }
            }
            // Da li moramo dodati novi meni?
            if (Add)
            {
                ToolStripMenuItem mi =
                    new ToolStripMenuItem(colorizer.ToString(), null, mnuViewTypes_Click);
                mi.CheckOnClick = true;
                mi.CheckState = CheckState.Unchecked;
                mi.Tag = colorizer;
                if (colorizer.IsSensor)
                {
                    mnuViewTypes.DropDownItems.Insert(mnuViewTypes.DropDownItems.IndexOf(mnuSenzorSeparator) + 1, mi);
                }
                else
                {
                    mnuViewTypes.DropDownItems.Insert(mnuViewTypes.DropDownItems.IndexOf(mnuSenzorSeparator), mi);
                }
                mi.PerformClick();
            }
        }
        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
		private void btnDisplayOptions_Click(object sender, EventArgs e)
		{
			switch (Convert.ToInt32(btnDisplayOptions.Tag))
			{
				case 0:
					while (tabPFragSelect.TabPages.Count > 0)
					{
						tabPFragSelect.TabPages.RemoveAt(0);
					}
					tabPFragSelect.TabPages.Add(tabPFragOptions);

					btnDisplayOptions.Text = "Ubaci program";
					btnDisplayOptions.Tag = 1;
					btnDisplayOptions.Enabled = false;

					// Prikazujemo opcije
					FriendlyType pfrag = (FriendlyType)processFragments.SelectedNode.Tag;

					// determine if it is neccesary to load the settings form
					bool showForm = false;
					PropertyInfo[] properties = pfrag.InnerType.GetProperties();
					for (int i = 0; i < properties.Length; i++)
						if (properties[i].GetCustomAttributes(typeof (BrowsableAttribute), false).Length != 0)
							showForm = true;

					// create a pfrag instance
					stateVectorProperties.SelectedObject = Activator.CreateInstance(pfrag, false);

					// show this form (if neccesary and load it's grid with ProcessFragment's properties
					if (showForm)
					{
						stateVectorProperties.Visible = true;
					}
					else
					{
						stateVectorProperties.Visible = false;
					}
					break;

					// Ovdje se vri invejdanje
				case 1:

					simulator.InvadeComputer((int) txtInvadeThisPC.Tag,
					                         (ProcessFragmentBase) stateVectorProperties.SelectedObject);

					//Makni tab
					DoRemoveTab(tabPFrags);
					EnableGroup(DisabledGroups.SimulationStarted);
					panDisplay.Cursor = Cursors.Default;
					internalForceDraw = true;

                    AddColorizerToMenu((FriendlyType) processFragments.SelectedNode.Tag);

					break;
			}
		}

		private void mnuShowCurrentScreen_Click(object sender, EventArgs e)
		{
			mnuDoShowScreen.Checked = true;
		}

		private void mnuDeleteRegion_Click(object sender, EventArgs e)
		{
			btnMassKillExecute.Enabled = false;
			if (mnuDeleteRegion.Checked)
			{
				MouseClickDoes = MouseClick.TransparentRegion;
				panDisplay.Cursor = Cursors.Cross;
				DoAddTab(tabMassKill);
			}
			else
			{
				MouseClickDoes = MouseClick.Nothing;
				panDisplay.Cursor = Cursors.Default;
				DoRemoveTab(tabMassKill);
			}
			EnableGroup(DisabledGroups.SimulationStarted);
		}

		private void processFragments_AfterSelect(object sender, TreeViewEventArgs e)
		{
			FriendlyType t = (FriendlyType)processFragments.SelectedNode.Tag;


            if (t != null)
                txtFragDescription.Text = processFragments.SelectedNode.ToolTipText;
            else
                txtFragDescription.Text = "";

			btnDisplayOptions.Enabled = false;
			if (processFragments.SelectedNode != null)
				btnDisplayOptions.Enabled = processFragments.SelectedNode.Level == 1;
		}

		/// <summary>
		/// Handles the Click event of the btnMassKillExecute control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="e">The <see cref="T:System.EventArgs"/> instance containing the event data.</param>
		private void btnMassKillExecute_Click(object sender, EventArgs e)
		{
            if (Convert.ToDouble(numDisableRandomPercentage.Value) > 0.00)
            {
                simulator.RemoveRandomComputers(Convert.ToDouble(numDisableRandomPercentage.Value));
            }
            else
            {
                simulator.RemoveComputersInPolygon(display.GetTransparentPoints());

                if (display != null)
                    display.AddTransparentPoint(Point.Empty, false);
            }

			btnMassKillCancel.PerformClick();
		}

		private void btnMassKillCancel_Click(object sender, EventArgs e)
		{
			MouseClickDoes = MouseClick.Nothing;
			mnuDeleteRegion.Checked = false;
			DoRemoveTab(tabMassKill);
			if (display != null)
				display.AddTransparentPoint(Point.Empty, false);
			EnableGroup(DisabledGroups.SimulationStarted);
		}

		private void btmSelectPortal_Click(object sender, EventArgs e)
		{
			panDisplay.Cursor = Cursors.NoMove2D;
			MouseClickDoes = MouseClick.InfectSelect;
		}

		private void txtInvadeThisPC_TextChanged(object sender, EventArgs e)
		{
			btnDisplayOptions.Enabled = (txtInvadeThisPC.Text != "");
		}

		//
		private void mnuViewTypes_Click(object sender, EventArgs e)
		{
			if (((ToolStripMenuItem) sender).Tag == null) return;

			for (int i = 0; i < mnuViewTypes.DropDownItems.Count; i++)
				((ToolStripMenuItem) mnuViewTypes.DropDownItems[i]).Checked = false;

			((ToolStripMenuItem) sender).Checked = true;

            if (((ToolStripMenuItem)sender).Tag.GetType() == typeof(FriendlyType))
                internalColorizer = ((FriendlyType)((ToolStripMenuItem)sender).Tag).DefaultColorizer;
		}

		private void mnuShowInfo_Click(object sender, EventArgs e)
		{
			if (infoThreadSynchro != null)
				infoThreadSynchro.DoCallDelegate = mnuShowInfo.Checked;
		}

		private void panDisplay_Resize(object sender, EventArgs e)
		{
            if (display == null) return;
            display.RefreshDisplaySize();
            CalculateScrollbars();
		}

		private void btnSetAviFile_Click(object sender, EventArgs e)
		{
			if (dialogSaveAvi.ShowDialog() == DialogResult.OK)
			{
				txtAVIOut.Text = dialogSaveAvi.FileName;
			}
		}


		private void aVIToolStripMenuItem_Click(object sender, EventArgs e)
		{
			for (int i = 0; i < mnuAviExport.DropDownItems.Count; i++)
				if (mnuAviExport.DropDownItems[i].GetType() == typeof (ToolStripMenuItem))
				{
					((ToolStripMenuItem) mnuAviExport.DropDownItems[i]).Checked = false;
				}

			((ToolStripMenuItem) sender).Checked = true;
			mnuAviExport.Tag = ((ToolStripMenuItem) sender).Tag;
		}

		private void txtWirelesRadius_ValueChanged(object sender, EventArgs e)
		{
			if (propertyEngineSettings.SelectedObject == null) return;
			((EngineSettings) propertyEngineSettings.SelectedObject).MaximumComputerCount = Convert.ToUInt32(txtbrojPC.Value);
			((EngineSettings) propertyEngineSettings.SelectedObject).WirelessRadius = Convert.ToUInt32(txtWirelessRadius.Value);
			propertyEngineSettings.Refresh();
		}

		private void propertyEngineSettings_PropertyValueChanged(object s, PropertyValueChangedEventArgs e)
		{
			txtbrojPC.Value = ((EngineSettings) propertyEngineSettings.SelectedObject).MaximumComputerCount;
			txtWirelessRadius.Value = ((EngineSettings) propertyEngineSettings.SelectedObject).WirelessRadius;
		}


        private void zoomToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (((ToolStripMenuItem)sender).Tag == null) return;

            foreach (ToolStripMenuItem tsmi in zoomToolStripMenuItem.DropDownItems)
            {
                if (tsmi == sender)
                    tsmi.Checked = true;
                else
                    tsmi.Checked = false;
            }
            if (display == null) return;
            CalculateScrollbars();
        }

        private void mnuNewDefaultSimulation_Click(object sender, EventArgs e)
        {
            // Simuliramo ono sto bi korisnik kliknuo
            mnuNewSimulation_Click(null, EventArgs.Empty);
            btnDoStartSimulation_Click(null, EventArgs.Empty);
        }

        private void mnuInsertSenzor_Click(object sender, EventArgs e)
        {
            // Popunimo listu moguih senzora
            twSenzors.Nodes.Clear();
            lwSenzersToAdd.Items.Clear();
            btnInsertSensors.Enabled = false;

            SensorTypes.Clear();
            SensorLocations.Clear();


            numRandomSenzorsCount.Minimum = 0;
            numRandomSenzorsCount.Maximum = 0;
            if (simulator != null)
            {
                List<FriendlyType> senzors = simulator.Plugins.ValidSensors;
                for (int i = 0; i < senzors.Count; i++)
                {
                    ISensor s = (ISensor) Activator.CreateInstance(senzors[i]);
                    TreeNode senzor = twSenzors.Nodes.Add(senzors[i].UserDefinedName);
                    senzor.Tag = s;
                    //senzor.Nodes.Add(s.SensorValueTypesRequired.ToString());
					for (int j = 0; j < s.DataFeeders.Length; j++)
						senzor.Nodes.Add(s.DataFeeders[j].Name);
                }

                numRandomSenzorsCount.Maximum = simulator.SimulatorStatus.Environment.WallSize.Height * simulator.SimulatorStatus.Environment.WallSize.Height
                                                - simulator.SimulatorStatus.ComputerCount;
            }
            //
            DoAddTab(tabSenzor);
            EnableGroup(DisabledGroups.SimulationStarted);
        }


        private void btnCancelInsertSenzor_Click(object sender, EventArgs e)
        {
            DoRemoveTab(tabSenzor);
            EnableGroup(DisabledGroups.SimulationStarted);

            panDisplay.Cursor = Cursors.Default;
            MouseClickDoes = MouseClick.Nothing;
        }

        private void twSenzors_AfterSelect(object sender, TreeViewEventArgs e)
        {
            panDisplay.Cursor = Cursors.Default;
            MouseClickDoes = MouseClick.Nothing;
            btnInsertSensors.Enabled = false;

            if (twSenzors.SelectedNode == null) return;
            if (twSenzors.SelectedNode.Level == 0)
            {
                panDisplay.Cursor = Cursors.NoMove2D;
                MouseClickDoes = MouseClick.SenzorAdd;

                btnInsertSensors.Enabled = (numRandomSenzorsCount.Value > 0);
            }
        }

        private void btnInsertSensors_Click(object sender, EventArgs e)
        {
            if (simulator == null) return;

            if (tabSensorInsertType.SelectedTab == tabRandomInsertSensors)
            {
                Random rnd = new Random();
                SensorTypes.Clear();
                SensorLocations.Clear();
                for (int i = 0; i < numRandomSenzorsCount.Value; i++)
                {
                    SensorTypes.Add(twSenzors.SelectedNode.Tag.GetType());
                    Point tmp = new Point(rnd.Next(simulator.SimulatorStatus.Environment.WallSize.Width - (int)simulator.SimulationSettings.SpaceBetweenPaintableComputers),
                                          rnd.Next(simulator.SimulatorStatus.Environment.WallSize.Height - (int)simulator.SimulationSettings.SpaceBetweenPaintableComputers));
                    while (simulator.GetComputerInfoFromPoint(tmp) != null)
                    {
                        tmp = new Point(rnd.Next(simulator.SimulatorStatus.Environment.WallSize.Width - (int)simulator.SimulationSettings.SpaceBetweenPaintableComputers),
                                        rnd.Next(simulator.SimulatorStatus.Environment.WallSize.Height - (int)simulator.SimulationSettings.SpaceBetweenPaintableComputers));
                    }
                    SensorLocations.Add(tmp);
                }
            }

            // Dodajmo u simulator
            simulator.InsertSensors(SensorLocations, SensorTypes);
            //dodajmo colorizere u GUI
            for (int i = 0; i < SensorTypes.Count; i++)
                AddColorizerToMenu(SensorTypes[i]);
            btnCancelInsertSenzor.PerformClick();
        }

        #endregion OnClick Events

        private void hSplit_SplitterMoved(object sender, SplitterEventArgs e)
        {

        }

        private void tabPage1_Click(object sender, EventArgs e)
        {

        }

        private void tabOutput_SelectedIndexChanged(object sender, EventArgs e)
        {

        }

        private void mnuScreenShot_Click(object sender, EventArgs e)
        {
            if (display == null) return;
            string path = Application.ExecutablePath;
            path = path.Substring(0, path.LastIndexOf("\\")) + "\\ScreenShots\\";
            try
            {
                Directory.CreateDirectory(path);
                Bitmap bmp = display.ReturnBitmap();
                bmp.Save(path + "Screen "+ DateTime.Now.Ticks.ToString() + ".bmp");
            }
            catch
            { };
        }

        private void tabOpcije_SelectedIndexChanged(object sender, EventArgs e)
        {

        }

        private void tabMassKill_Click(object sender, EventArgs e)
        {

        }

        private void numDisableRandomPercentage_ValueChanged(object sender, EventArgs e)
        {
            btnMassKillExecute.Enabled = Convert.ToDouble(numDisableRandomPercentage.Value) > 0;
        }

        private void menuMain_ItemClicked(object sender, ToolStripItemClickedEventArgs e)
        {

        }

        private void frmGUI_Load(object sender, EventArgs e)
        {

        }

        private void tabControl3_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (tabSensorInsertType.SelectedTab != tabRandomInsertSensors)
                btnInsertSensors.Enabled = lwSenzersToAdd.Items.Count > 0;
            else
                if (twSenzors.SelectedNode == null)
                    btnInsertSensors.Enabled = false;
                else
                    btnInsertSensors.Enabled = (numRandomSenzorsCount.Value>0) && (twSenzors.SelectedNode.Level == 0);
        }

        private void mnuBoldSensorsAndPortals_Click(object sender, EventArgs e)
        {
            if (display == null) return;
            display.BoldSensorsAndPortals = mnuBoldSensorsAndPortals.Checked;
        }

        private void numRandomSenzorsCount_ValueChanged(object sender, EventArgs e)
        {
            if (twSenzors.SelectedNode == null)
                btnInsertSensors.Enabled = false;
            else
                btnInsertSensors.Enabled = (numRandomSenzorsCount.Value > 0) && (twSenzors.SelectedNode.Level == 0);
        }
	}


	/// <summary>
	/// 
	/// </summary>
	public class InfoThreadSychronization
	{
		public bool DoWork;
		public bool DoCallDelegate;

		public InfoThreadSychronization()
		{
			DoWork = true;
			DoCallDelegate = true;
		}
	}

	/// <summary>
	/// 
	/// </summary>
	public class InfoThread
	{
		private frmGUI baseForm;
		private InfoThreadSychronization info;

		/// <summary>
		/// 
		/// </summary>
		/// <param name="form"></param>
		/// <param name="infoSychronization"></param>
		public InfoThread(frmGUI form, ref InfoThreadSychronization infoSychronization)
		{
			baseForm = form;
			info = infoSychronization;
		}

		/// <summary>
		/// 
		/// </summary>
		public void Run()
		{
			bool DoRun;
			try
			{
				do
				{
					if (info.DoCallDelegate)
					{
						lock (info)
						{
							baseForm.Invoke(baseForm.doShowInfo);
						}
					}
					DoRun = info.DoWork;
					Thread.Sleep(20);
				} while (DoRun);
			}
			catch (Exception e)
			{
			}
			;
		}
	}

	#region Display FPS

	internal static class FPS
	{
		public static DateTime LastUpdateTime;

		private static UInt64 LastTickCount;
		private static uint fFPS = 0;

		public static uint GetFPS(UInt64 TickCount)
		{
			DateTime tmp = DateTime.Now;

			// Prolo vie od sekunde
			Single diff = Convert.ToSingle((tmp.Ticks - LastUpdateTime.Ticks)/10000000F);
			if (diff >= 1)
			{
				if (LastTickCount > TickCount)
					LastTickCount = 0;
				fFPS = Convert.ToUInt32(((TickCount - LastTickCount)/diff));

				LastUpdateTime = tmp;
				LastTickCount = TickCount;
			}


			return fFPS;
		}

		public static bool SecondGone()
		{
			DateTime tmp = DateTime.Now;
			Single diff = Convert.ToSingle((tmp.Ticks - LastUpdateTime.Ticks)/10000000F);
			return (diff > 1.0);
		}
	}

	#endregion Display FPS
}