/*
	Paintable Computing Simulation

	License

	This software (Paintable Computing Simulation and all parts of its 
	source code) is provided "as is", without any warranty and it is 
	the property of its respective authors. Use of this software is 
	free to everyone, except in commercial and/or military purposes 
	without written permission of its authors. Disassembling the 
	executable code of this software is forbidden for any purpose 
	without written permission of its authors.

	If you intend to use any part of the source code of this software 
	or to develop this software any further, you must include this 
	license in every source file and every About box, without changing 
	it; only appending the license is allowed. And if you use any part 
	of the source code of this software, or develop it further, please 
	send an e-mail to its authors.

	Authors of this software are not responsible for any consequences 
	which use of this software, use of any part of its source or its 
	further development may cause.

	All software and parts of source code used in the development of 
	this software are property of their respective owners.

	Good luck,
	Authors

	Authors (alphabetically):
	Fabijan Lukin
		fabijan@gmail.com
		fabijan.lukin@fer.hr
	Fran Pregernik
		fran.pregernik@gmail.com
		fran.pregernik@fer.hr
	Tomislav Sukser
		tomislav.sukser@gmail.com
		tomislav.sukser@zg.t-com.hr
		tomislav.sukser@fer.hr


	Note:
	Paintable computing was firstly introduced in Programming a 
	Paintable Computer written by William Joseph Butera.


    Copyright (c) 2005,2006. All rights reserved.
    4-6/2006

 */
using System;
using System.Collections.Generic;
using System.Text;

namespace PaintableSimulator.Engine
{
	internal static class EngineLog
	{
		// Create a logger for use in this class
		private static readonly log4net.ILog log = log4net.LogManager.GetLogger("ENGINE");
		// NOTE that using System.Reflection.MethodBase.GetCurrentMethod().DeclaringType
		// is equivalent to typeof(LoggingExample) but is more portable
		// i.e. you can copy the code directly into another class without
		// needing to edit the code.

		#region ILog Members

		public static void Debug(object message)
		{
			log.Debug(message);
		}

		public static void Debug(object message, Exception exception)
		{
			log.Debug(message, exception);
		}

		public static void DebugFormat(string format, params object[] args)
		{
			log.DebugFormat(format, args);
		}

		public static void DebugFormat(IFormatProvider provider, string format, params object[] args)
		{
			log.DebugFormat(provider, format, args);
		}

		public static void Info(object message)
		{
			log.Info(message);
		}

		public static void Info(object message, Exception exception)
		{
			log.Info(message, exception);
		}

		public static void InfoFormat(string format, params object[] args)
		{
			log.InfoFormat(format, args);
		}

		public static void InfoFormat(IFormatProvider provider, string format, params object[] args)
		{
			log.InfoFormat(provider, format, args);
		}

		public static void Warn(object message)
		{
			log.Warn(message);
		}

		public static void Warn(object message, Exception exception)
		{
			log.Warn(message, exception);
		}

		public static void WarnFormat(string format, params object[] args)
		{
			log.WarnFormat(format, args);
		}

		public static void WarnFormat(IFormatProvider provider, string format, params object[] args)
		{
			log.WarnFormat(provider, format, args);
		}

		public static void Error(object message)
		{
			log.Error(message);
		}

		public static void Error(object message, Exception exception)
		{
			log.Error(message, exception);
		}

		public static void ErrorFormat(string format, params object[] args)
		{
			log.ErrorFormat(format, args);
		}

		public static void ErrorFormat(IFormatProvider provider, string format, params object[] args)
		{
			log.ErrorFormat(provider, format, args);
		}

		public static void Fatal(object message)
		{
			log.Fatal(message);
		}

		public static void Fatal(object message, Exception exception)
		{
			log.Fatal(message, exception);
		}

		public static void FatalFormat(string format, params object[] args)
		{
			log.FatalFormat(format, args);
		}

		public static void FatalFormat(IFormatProvider provider, string format, params object[] args)
		{
			log.FatalFormat(provider, format, args);
		}

		public static bool IsDebugEnabled
		{
			get { return log.IsDebugEnabled; }
		}

		public static bool IsInfoEnabled
		{
			get { return log.IsInfoEnabled; }
		}

		public static bool IsWarnEnabled
		{
			get { return log.IsWarnEnabled; }
		}

		public static bool IsErrorEnabled
		{
			get { return log.IsErrorEnabled; }
		}

		public static bool IsFatalEnabled
		{
			get { return log.IsFatalEnabled; }
		}

		#endregion

		#region ILoggerWrapper Members

		public static log4net.Core.ILogger Logger
		{
			get { return log.Logger; }
		}

		#endregion
	}
}
