/*
	Paintable Computing Simulation

	License

	This software (Paintable Computing Simulation and all parts of its 
	source code) is provided "as is", without any warranty and it is 
	the property of its respective authors. Use of this software is 
	free to everyone, except in commercial and/or military purposes 
	without written permission of its authors. Disassembling the 
	executable code of this software is forbidden for any purpose 
	without written permission of its authors.

	If you intend to use any part of the source code of this software 
	or to develop this software any further, you must include this 
	license in every source file and every About box, without changing 
	it; only appending the license is allowed. And if you use any part 
	of the source code of this software, or develop it further, please 
	send an e-mail to its authors.

	Authors of this software are not responsible for any consequences 
	which use of this software, use of any part of its source or its 
	further development may cause.

	All software and parts of source code used in the development of 
	this software are property of their respective owners.

	Good luck,
	Authors

	Authors (alphabetically):
	Fabijan Lukin
		fabijan@gmail.com
		fabijan.lukin@fer.hr
	Fran Pregernik
		fran.pregernik@gmail.com
		fran.pregernik@fer.hr
	Tomislav Sukser
		tomislav.sukser@gmail.com
		tomislav.sukser@zg.t-com.hr
		tomislav.sukser@fer.hr


	Note:
	Paintable computing was firstly introduced in Programming a 
	Paintable Computer written by William Joseph Butera.


    Copyright (c) 2005,2006. All rights reserved.
    4-6/2006

 */
#region Using directives

using System;
using System.ComponentModel;
using System.Drawing;
using PaintableSimulator.Engine;
using PaintableSimulator.Engine.PaintableComputers.OperatingSystem;
using System.Collections.Generic;
using PaintableSimulator.Engine.PaintableComputers;

#endregion

namespace PaintableSimulator.Programs
{

	#region Colorizer

	/// <summary>
	/// 
	/// </summary>
	public class CoordinateTunnelColorizer : HomepageColorizer
	{
		private static List<double> maximumReachedDistances = new List<double>();

		Color[] pallete = new Color[] { Color.LightGreen, Color.Red, Color.Magenta, Color.Yellow, Color.Cyan };


		public CoordinateTunnelColorizer()
		{

			for (int i = 0; i < pallete.Length; i++)
				maximumReachedDistances.Add(0.0);
		}
		/// <summary>
		/// Colorizes the specified homepage.
		/// </summary>
		/// <param name="homepage">The homepage.</param>
		/// <returns></returns>
		public override Color Colorize(Homepage homepage)
		{
			Color retval = Color.WhiteSmoke;

			try
			{
				for (int i = 0; i < homepage.Count; i++)
					if (homepage.KeyList[i] == CoordinateTunnel.GradientId)
					{
						double minDistance = double.MaxValue;
						int minDistanceIndex = 0;

						for (int j = 0; (j < 3) && (j < homepage.DataList[i].Count); j++)
						{
							if (homepage.DataList[i][j] == null)
							{
								continue;
							}
							double distance = ((double[])homepage.DataList[i][j])[3];

							// find the minimal distance of all local gradients
							if (minDistance > distance)
							{
								minDistance = distance;
								minDistanceIndex = j;
							}

							// store the maximum distance
							if (maximumReachedDistances[j] < distance)
							{
								maximumReachedDistances[j] = distance;
							}
						}

						Color baseColor = Color.Green;
						if (minDistanceIndex < pallete.Length)
							baseColor = pallete[minDistanceIndex];

						double modifier = ((1 - minDistance / (maximumReachedDistances[minDistanceIndex] + 0.001)));

						retval = Color.FromArgb(255, (int)(baseColor.R * modifier), (int)(baseColor.G * modifier), (int)(baseColor.B * modifier));
					}
			}
			catch
			{
				return retval;
			}
			return retval;
		}
	}

	#endregion

	public enum GradientType
	{
		First,
		Second,
		Third

	};

	/// <summary>
	/// 
	/// </summary>
	[Serializable()]
	[DefaultHomepageColorizerAttribute(typeof(CoordinateTunnelColorizer))]
	[FriendlyName("Coordinate Tunnel")]
	public sealed class CoordinateTunnel : ProcessFragmentBase
	{
		#region Process Fragment Identity

		public const uint GradientId = 0x2890FB4c;

		public override uint ProcessFragmentId
		{
			get { return GradientId; }
		}

		#endregion

		#region Gradient Local state

		private int _gradientId1 = new Random().Next(0, 100);
		private int _gradientId2 = new Random().Next(0, 100);
		private int _hopCount;
		private double _distance;

		/// <summary>
		/// Gets or sets the gradient id1.
		/// </summary>
		/// <value>The gradient id1.</value>
		[Browsable(true)]
		[Category("Portal ID's")]
		public int GradientId1
		{
			get { return _gradientId1; }
			set { _gradientId1 = value; }
		}

		/// <summary>
		/// Gets or sets the gradient id2.
		/// </summary>
		/// <value>The gradient id2.</value>
		[Browsable(true)]
		[Category("Portal ID's")]
		public int GradientId2
		{
			get { return _gradientId2; }
			set { _gradientId2 = value; }
		}


		/// <summary>
		/// Gets or sets the hop count.
		/// </summary>
		/// <value>The hop count.</value>
		[Browsable(false)]
		public int HopCount
		{
			get { return _hopCount; }
			set { _hopCount = value; }
		}

		/// <summary>
		/// Gets or sets the distance.
		/// </summary>
		/// <value>The distance.</value>
		[Browsable(false)]
		public double Distance
		{
			get { return _distance; }
			set { _distance = value; }
		}

		private GradientType tip;
		[Browsable(true)]
		public GradientType Tip
		{
			get { return tip; }
			set
			{
				tip = value;
				switch (tip)
				{
					case GradientType.First:
					case GradientType.Third:
						startXY.X = 0;
						startXY.Y = 0;
						break;
					case GradientType.Second:
						startXY.X = 5;
						startXY.Y = 1;
						break;
				}
			}
		}

		private Point startXY;
		[Browsable(true)]
		public Point StartPoint
		{
			get { return startXY; }
			set { startXY = value; }
		}
		private bool isPortal = true;

		#endregion

		/// <summary>
		/// This method is called when the PFrag first enters a Paintable.
		/// </summary>
		/// <remarks>
		/// </remarks>
		public override void Install()
		{
			if (Api.GetPostCount(this.ProcessFragmentId) != 0)
			{
				// if the homepage is not empty, check if there is a post with the corresponding gradient ID's
				for (int j = 0; (j < 3) && (j < Api.GetPostCount(this.ProcessFragmentId)); j++)
				{
					// read post j
					double[] postValues = (double[])Api.GetPost(this.ProcessFragmentId, j);
					if (postValues == null)
						continue;

					// get the ID's
					int gID1 = (int)postValues[0];
					int gID2 = (int)postValues[1];

					// compare them to the one this instance has
					if ((gID1 == this.GradientId1) && (gID2 == this.GradientId2))
					{
						// there is already on of them with the same gradient ID's, so uninstall
						Api.QueueUninstall();
						return;
					}
				}
			}

			// everything is ok, we are allowed to install

			// prepare post
			double[] post = new double[] {this.GradientId1,
                                          this.GradientId2,
                                          this.HopCount,
                                          this.Distance,
                                          this.startXY.X,
                                          this.startXY.Y};

			// post
			switch (this.tip)
			{
				case GradientType.First:
					Api.PostEntry(this.ProcessFragmentId, 0, post); // at index
					break;
				case GradientType.Second:
					Api.PostEntry(this.ProcessFragmentId, 1, post); // at index
					break;
				case GradientType.Third:
					Api.PostEntry(this.ProcessFragmentId, 2, post); // at index
					break;
			}
		}


		/// <summary>
		/// This method is called after the PFrag queued himself for uninstall and the OS approves an uninstall.
		/// </summary>
		/// <param name="remainingTimeslots"></param>
		/// <remarks>
		/// Here you should do some pre-uninstall processing.
		/// Removing posts from the homepage is unnecessary because the OS does that for you automatically.
		/// </remarks>
		public override void Uninstall(byte remainingTimeslots)
		{
			// remove posts from this instance
			// if the homepage is not empty, check if there is a post with the corresponding gradient ID's
			for (int j = 0; (j < 3) && (j < Api.GetPostCount(this.ProcessFragmentId)); j++)
			{
				// read post j
				double[] postValues = (double[])Api.GetPost(this.ProcessFragmentId, j);
				if (postValues == null)
					continue;

				// get the ID's
				int gID1 = (int)postValues[0];
				int gID2 = (int)postValues[1];

				// compare them to the one this instance has
				if ((gID1 == this.GradientId1) && (gID2 == this.GradientId2))
				{
					Api.RemoveEntry(this.ProcessFragmentId, j);
				}
			}
		}


		/// <summary>
		/// This method is called if the PFrag requested a transfer to the neighboring particles and
		/// his request was approved.
		/// </summary>
		/// <param name="copy">The copy of the ProcessFragmentBase.</param>
		public override void TransferGranted(ref ProcessFragmentBase copy)
		{
			CoordinateTunnel gradientCopy = (CoordinateTunnel)copy;


			gradientCopy.tip = this.tip;
			gradientCopy.isPortal = false;
			gradientCopy.GradientId1 = this.GradientId1;
			gradientCopy.GradientId2 = this.GradientId2;
			gradientCopy.Distance = 0.0;
			gradientCopy.HopCount = this.HopCount + 1;
			gradientCopy.startXY = this.startXY;
		}


		/// <summary>
		/// This method is called if the PFragment requested a transfer to the neighboring particles and
		/// his request was denied.
		/// </summary>
		public override void TransferRefused()
		{
		}

		/// <summary>
		/// The main processing method for Process Fragments
		/// </summary>
		/// <param name="remainingTimeslots">A value indicating the available processing time for the Update method.</param>
		/// <remarks>
		/// The time slot of 0 indicates less time available for processing and 255 more time.
		/// This is jst a suggested value, it has no bearing on the execution of the Update method (yet).
		/// </remarks>
		public override void Update(byte remainingTimeslots)
		{
			//Propagation
			// check for PaC's without Gradients and INVADE
			int mirrorCount = Api.GetIOSpaceCount();
			for (int i = 0; i < mirrorCount; i++)
			{
				// check for empty Pac's
				if (Api.GetIOPostCount(i, ProcessFragmentId) == 0)
				{
					if (!Api.IsQueuedForTransfer(i))
					{
						Api.QueueTransfer(i);
						continue;
					}
				}

				// if they are not empty, check if there is a post with the corresponding gradient ID's
				bool notInstalled = true;
				for (int j = 0; (j < 3) && (j < Api.GetIOPostCount(i, this.ProcessFragmentId)); j++)
				{
					// read post j
					double[] postValues = (double[])Api.GetIOPost(i, this.ProcessFragmentId, j);
					if (postValues == null)
						continue;

					// get the ID's
					int gID1 = (int)postValues[0];
					int gID2 = (int)postValues[1];

					// compare them to the one this instance has
					if ((gID1 == this.GradientId1) && (gID2 == this.GradientId2))
					{
						notInstalled = false;
						break;
					}
				}

				if (notInstalled)
					Api.QueueTransfer(i);
			}

			// if the PFrag is queued for transfer, ignore further processing
			if (Api.IsQueuedForTransfer()) return;


			//scan mirror home pages
			int hopCountMinimum = this.HopCount;
			int sum = 0;
			for (int i = 0; i < mirrorCount; i++)
				for (int j = 0; (j < 3) && (j < Api.GetIOPostCount(i, this.ProcessFragmentId)); j++)
				{
					// read post j
					double[] postValues = (double[])Api.GetIOPost(i, this.ProcessFragmentId, j);
					if (postValues == null)
						continue;

					// get the ID's
					int gID1 = (int)postValues[0];
					int gID2 = (int)postValues[1];

					// compare them to the one this instance has
					if ((gID1 == this.GradientId1) && (gID2 == this.GradientId2))
					{
						// ID's match, sum up the hops
						int hops = (int)postValues[2];

						sum += hops;
						if (hops < hopCountMinimum)
							hopCountMinimum = hops;
					}
				}


			//Adaptation
			if (hopCountMinimum <= this.HopCount)
			{
				//calculate average distance
				// ATB: dodan IF - ako imamo veci hopcount od novog najmanjeg
				//        onda bi ga trebali proglasiti besmislenim i uzeti ovaj
				//        manji (obrnuto razmisljanje nego za novce) :)
				if (this.HopCount > hopCountMinimum)
					this.HopCount = hopCountMinimum + 1;

				sum += this.HopCount;
				this.Distance = ((double)sum) / ((double)(mirrorCount + 1));

				//get the index of the post with the correct gradient ID's
				for (int j = 0; (j < 3) && (j < Api.GetPostCount(this.ProcessFragmentId)); j++)
				{
					// read post j
					double[] postValues = (double[])Api.GetPost(this.ProcessFragmentId, j);
					if (postValues == null)
						continue;

					// get the ID's
					int gID1 = (int)postValues[0];
					int gID2 = (int)postValues[1];

					// compare them to the one this instance has
					if ((gID1 == this.GradientId1) && (gID2 == this.GradientId2))
					{
						// prepare post
						double[] post = new double[] {  this.GradientId1,
                                                        this.GradientId2,
                                                        this.HopCount,
                                                        this.Distance,
                                                        this.startXY.X,
                                                        this.startXY.Y};

						// repost
						Api.PostEntry(this.ProcessFragmentId, j, post);

						break;
					}
				}
			}
			else
			{
				//removal -- queue for uninstall
				Api.QueueUninstall();
			}

			switch (this.tip)
			{
				case GradientType.Second:
					if (Api.GetPostCount() == 2)
					{
						double r = double.MaxValue;
						if (this.isPortal)
						{
							Api.PostEntry(this.ProcessFragmentId, 3, ((double[])Api.GetPost(this.ProcessFragmentId, 0))[2]);
						}
						else
						{
							mirrorCount = Api.GetIOSpaceCount();
							for (int i = 0; i < mirrorCount; i++)
							{
								// Ovo ne radi?!?
								if (Api.GetIOPostCount(i, this.ProcessFragmentId) >= 3)
								{
									Object x = Api.GetIOPost(i, this.ProcessFragmentId, 3);
									if (x != null)
										if (((double)x) < r)
											r = (double)x;
								}
							}
							if (r < double.MaxValue)
								Api.PostEntry(3, r);
						}
					}
					break;
				case GradientType.Third:
					// Mozemo racunati Gamu i proslijediti je uokolo
					if (Api.GetPostCount() == 4)
					{
						if (this.isPortal)
						{
                            /*
                                a[3] ti je radijus od 1 tocke, a[4] je X1, a[5] je y1...
                                isti princim je za b[]...
                            
                                hopX ti je koliko hopova treba skociti da bi se x pomaknuo za 1...
                            */
                            double[] a = (double[])Api.GetPost(0);
							double[] b = (double[])Api.GetPost(1);
							double c = (double)Api.GetPost(3);
                            
							double hopX = Math.Sqrt(Math.Pow(a[4] - b[4], 2) + Math.Pow(a[5] - b[5], 2)) / c;
                            /*
							// Racunaj gamu
                             */
							double gama = ((a[2] * hopX * a[2] * hopX) + (c * hopX * c * hopX) - (b[2] * hopX * b[2] * hopX)) / (2 * a[2] * hopX * c * hopX);
							Api.PostEntry(this.ProcessFragmentId, 4, Math.Acos(gama));
                            
							Api.PostEntry(this.ProcessFragmentId, 5, hopX);
                           
							double[] post = (double[])Api.GetPost(2);
                            /*
							post[4] = a[2] * hopX * gama;
							post[5] = a[2] * hopX * Math.Sin(Math.Acos(gama));
							*/


                            double i1, i2, j1, j2, d1, d2;
                            double x, y, x1, x2;

                            if (a[4] - b[4] == 0)
                            {
                                // Just swap a and b
                                i1 = b[5];
                                i2 = a[5];
                                j1 = b[4];
                                j2 = a[4];
                                d1 = b[2]*hopX*1.02;
                                d2 = a[2]*hopX*1.02;
                            }
                            else// if (p1.Y - p2.Y == 0)
                            {
                                // Normal case
                                i1 = a[4];
                                i2 = b[4];
                                j1 = a[5];
                                j2 = b[5];
                                d1 = a[2]*hopX*1.02;
                                d2 = b[2]*hopX*1.02;
                            }


                            double A = d1 * d1 - d2 * d2 + (i2 - i1) * (i2 - i1) + j2 * j2 - j1 * j1;
                            double B = 2 * (j1 - j2);
                            double C = 2 * (i1 - i2);

                            double R = d1 * d1 - j1 * j1 - (A / C) * (A / C);
                            double S = 2 * j1 - 2 * A * B / (C * C);
                            double T = -1 - (B / C) * (B / C);

                            // Y
                            double y1 = (-S + Math.Sqrt(S * S - 4 * R * T)) / (2 * T);
                            double y2 = (-S - Math.Sqrt(S * S - 4 * R * T)) / (2 * T);

                            // X
                            double x11 = i2 + Math.Sqrt(d2 * d2 - (y1 - j2) * (y1 - j2));
                            double x12 = i2 - Math.Sqrt(d2 * d2 - (y1 - j2) * (y1 - j2));

                            double x21 = i2 + Math.Sqrt(d2 * d2 - (y2 - j2) * (y2 - j2));
                            double x22 = i2 - Math.Sqrt(d2 * d2 - (y2 - j2) * (y2 - j2));


                            double r1 = Math.Abs(Math.Pow(x11 - i2, 2) + Math.Pow(y1 - j2, 2) - d2 * d2)
                                            + Math.Abs(Math.Pow(x11 - i1, 2) + Math.Pow(y1 - j1, 2) - d1 * d1);

                            double r2 = Math.Abs(Math.Pow(x12 - i2, 2) + Math.Pow(y1 - j2, 2) - d2 * d2)
                                            + Math.Abs(Math.Pow(x12 - i1, 2) + Math.Pow(y1 - j1, 2) - d1 * d1);


                            if (r1 <= r2)
                                x1 = x11;
                            else
                                x1 = x12;


                            r1 = Math.Abs(Math.Pow(x21 - i2, 2) + Math.Pow(y2 - j2, 2) - d2 * d2)
                                        + Math.Abs(Math.Pow(x21 - i1, 2) + Math.Pow(y2 - j1, 2) - d1 * d1);

                            r2 = Math.Abs(Math.Pow(x22 - i2, 2) + Math.Pow(y2 - j2, 2) - d2 * d2)
                                        + Math.Abs(Math.Pow(x22 - i1, 2) + Math.Pow(y2 - j1, 2) - d1 * d1);


                            if (r1 <= r2)
                                x2 = x21;
                            else
                                x2 = x22;

                            /*
                            r1 = Math.Abs(Math.Pow(x1 - i3, 2) + Math.Pow(y1 - j3, 2) - d3 * d3);
                            r2 = Math.Abs(Math.Pow(x2 - i3, 2) + Math.Pow(y2 - j3, 2) - d3 * d3);
                            */

                            r1 = Math.Abs(Math.Pow(x1 - i2, 2) + Math.Pow(y1 - j2, 2) - d2 * d2)
                                        + Math.Abs(Math.Pow(x1 - i1, 2) + Math.Pow(y1 - j1, 2) - d1 * d1);

                            r2 = Math.Abs(Math.Pow(x2 - i2, 2) + Math.Pow(y2 - j2, 2) - d2 * d2)
                                        + Math.Abs(Math.Pow(x2 - i1, 2) + Math.Pow(y2 - j1, 2) - d1 * d1);

                            
                            if (r1 <= r2 && y1 > y2)
                            {
                                x = x1;
                                y = y1;
                            }
                            else
                            {
                                x = x2;
                                y = y2;
                            }


                            if (a[4] - b[4] == 0)
                            {
                                post[5] = x;
                                post[4] = y;
                            }
                            else
                            {
                                post[4] = x;
                                post[5] = y;
                            }
                            // END ALGORITHM

                            this.startXY.X = Convert.ToInt32(post[4]);
							this.startXY.Y = Convert.ToInt32(post[5]);
							Api.PostEntry(this.ProcessFragmentId, 2, post);
						}
						else
						{
							mirrorCount = Api.GetIOSpaceCount();
							for (int i = 0; i < mirrorCount; i++)
							{
								if (Api.GetIOPostCount(i, this.ProcessFragmentId) > 4)
								{
									Api.PostEntry(4, Api.GetIOPost(i, this.ProcessFragmentId, 4));
									Api.PostEntry(5, Api.GetIOPost(i, this.ProcessFragmentId, 5));
								}
							}
						}
					}
					break;
			}


		}
	}


	#region Colorizer

	/// <summary>
	/// 
	/// </summary>
	public class CoordinateCalculatorColorizer : HomepageColorizer
	{
		private static List<double> maximumReachedDistances = new List<double>();

		Color[] pallete = new Color[] { Color.LightGreen, Color.Red, Color.Magenta, Color.Yellow, Color.Cyan, Color.Gray, Color.BlueViolet };


		/// <summary>
		/// Colorizes the specified homepage.
		/// </summary>
		/// <param name="homepage">The homepage.</param>
		/// <returns></returns>
		public override Color Colorize(Homepage homepage)
		{
			Color retval = Color.WhiteSmoke;
			Color baseColor = pallete[4];
			for (int i = 0; i < homepage.Count; i++)
				if (homepage.KeyList[i] == CoordinateCalculator.GradientId)
				{
					switch (homepage.DataList[i].Count)
					{
						case 0: return Color.DarkBlue;
						case 1: return Color.LemonChiffon;
						case 2:
                            int x = (int)homepage.DataList[i][0];
                            int y = (int)homepage.DataList[i][1];

                            if ((x < 0) || (y < 0))
                                return Color.White;
                            else
                            {
                                /* Sahovnica, ali bar i ne rsdi kako treba...
                                if (((x + y) % 2) == 0)
                                    return Color.Gray;
                                else
                                    return Color.Silver;
                                 */
                                return pallete[Convert.ToInt32(Math.Abs((x+y) % pallete.Length))];
                            }
					}
				}

			return retval;
		}
	}

	#endregion


	[Serializable()]
	[DefaultHomepageColorizerAttribute(typeof(CoordinateCalculatorColorizer))]
	[FriendlyName("Coordinate Calculator")]
	public sealed class CoordinateCalculator : ProcessFragmentBase
	{
		#region Process Fragment Identity

		public const uint GradientId = 0x2890FB4D;

		public override uint ProcessFragmentId
		{
			get { return GradientId; }
		}

		#endregion

		#region Gradient Local state
		#endregion

		// Precondition: one center point (p1/p2/p3) must not be colinear with the other two.
        private Point Trilaterate(Point p1, double d1, Point p2, double d2, Point p3, double d3)
        {



            //double S = (Math.Pow(i3,2) - Math.Pow(i2,2) + Math.Pow(j3,2) -
            //     Math.Pow(j2,2) + Math.Pow(d2,2) - Math.Pow(d3,2)) / 2.0;
            //double T = (Math.Pow(i1,2) - Math.Pow(i2,2) + Math.Pow(j1,2) -
            //     Math.Pow(j2,2) + Math.Pow(d2,2) - Math.Pow(d1,2)) / 2.0;

            //y = ((T*(i2-i3)) - (S*(i2-i1))) / (((j1-j2)*(i2-i3)) - ((j3-j2)*(i2-i1)));
            //x = ((y * (j1 - j2)) - T) / (i2 - i1);

            ////x = (((2 * j3 - 2 * j2) * ((d1 * d1 - d2 * d2) + (i2 * i2 - i1 * i1) + (j2 * j2 - j1 * j1))
            ////    - (2 * j2 - 2 * j1) * ((d2 * d2 - d3 * d3) + (i3 * i3 - i2 * i2) + (j3 * j3 - j2 * j2))) /
            ////        ((2 * i2 - 2 * i3) * (2 * j2 - 2 * j1) - (2 * i1 - 2 * i2) * (2 * j3 - 2 * j2)));
            ////y = ((d1 * d1 - d2 * d2) + (i2 * i2 - i1 * i1) + (j2 * j2 - j1 * j1) + x * (2 * i1 - 2 * i2)) / (2 * j2 - 2 * j1);

            Point eksvai = new Point();

            if (CalculatePoint(p1, d1, p2, d2, p3, d3, ref eksvai))
            {
                return eksvai;
            }
            else if (CalculatePoint(p1, d1, p3, d3, p2, d2, ref eksvai))
            {
                return eksvai;
            }
            else if (CalculatePoint(p3, d3, p2, d2, p1, d1, ref eksvai))
            {
                return eksvai;
            }
            else
            {
                return new Point(-1, -1);
            }

            /*
            try
            {
                return new Point(Convert.ToInt32(x), Convert.ToInt32(y));
            }
            catch
            {
            }
             * */
        }


        private bool CalculatePoint(Point p1, double d1, Point p2, double d2, Point p3, double d3, ref Point eksvai)
        {
            double i1, i2, i3, j1, j2, j3;
            double x, y, x1, x2;

            if (p1.X - p2.X == 0)
            {
                if (p1.X - p3.X == 0)
                {
                    return false;
                }

                // Just swap p2 and p3, d2 and d3
                // Normal case
                i1 = p3.X;
                i2 = p2.X;
                i3 = p1.X;
                j1 = p3.Y;
                j2 = p2.Y;
                j3 = p1.Y;

                double tempd = d1;
                d1 = d3;
                d3 = tempd;
            }
            else// if (p1.Y - p2.Y == 0)
            {
                // Normal case
                i1 = p1.X;
                i2 = p2.X;
                i3 = p3.X;
                j1 = p1.Y;
                j2 = p2.Y;
                j3 = p3.Y;
            }

            d1 = d1;
            d2 = d2;
            d3 = d3;

            double a = d1 * d1 - d2 * d2 + (i2 - i1) * (i2 - i1) + j2 * j2 - j1 * j1;
            double b = 2 * (j1 - j2);
            double c = 2 * (i1 - i2);

            double R = d1 * d1 - j1 * j1 - (a / c) * (a / c);
            double S = 2 * j1 - 2 * a * b / (c * c);
            double T = -1 - (b / c) * (b / c);

            // Y
            double y1 = (-S + Math.Sqrt(S * S - 4 * R * T)) / (2 * T);
            double y2 = (-S - Math.Sqrt(S * S - 4 * R * T)) / (2 * T);

            // X
            double x11 = i2 + Math.Sqrt(d2 * d2 - (y1 - j2) * (y1 - j2));
            double x12 = i2 - Math.Sqrt(d2 * d2 - (y1 - j2) * (y1 - j2));

            double x21 = i2 + Math.Sqrt(d2 * d2 - (y2 - j2) * (y2 - j2));
            double x22 = i2 - Math.Sqrt(d2 * d2 - (y2 - j2) * (y2 - j2));


            double r1 = Math.Abs(Math.Pow(x11 - i2, 2) + Math.Pow(y1 - j2, 2) - d2 * d2);
            double r2 = Math.Abs(Math.Pow(x12 - i2, 2) + Math.Pow(y1 - j2, 2) - d2 * d2);


            if (r1 < r2) // NE <=!!!!
                x1 = x11;
            else
                x1 = x12;


            r1 = Math.Abs(Math.Pow(x21 - i2, 2) + Math.Pow(y2 - j2, 2) - d2 * d2);
            r2 = Math.Abs(Math.Pow(x22 - i2, 2) + Math.Pow(y2 - j2, 2) - d2 * d2);


            if (r1 < r2) // NE <= !!!!
                x2 = x21;
            else
                x2 = x22;


            r1 = Math.Abs(Math.Pow(x1 - i3, 2) + Math.Pow(y1 - j3, 2) - d3 * d3);
            r2 = Math.Abs(Math.Pow(x2 - i3, 2) + Math.Pow(y2 - j3, 2) - d3 * d3);

            if (r1 < r2) // NE <= !!!!
            {
                x = x1;
                y = y1;
            }
            else
            {
                x = x2;
                y = y2;
            }

            try
            {
                eksvai.X = Convert.ToInt32(x);
                eksvai.Y = Convert.ToInt32(y);
                return true;
            }
            catch
            {
                return false;
            }
        }

		/// <summary>
		/// This method is called when the PFrag first enters a Paintable.
		/// </summary>
		/// <remarks>
		/// </remarks>
		public override void Install()
		{
			if (Api.GetPostCount(this.ProcessFragmentId) != 0)
			{
			}
			// everything is ok, we are allowed to install
			double hopX = (double)Api.GetPost(CoordinateTunnel.GradientId, 5);
			double a = ((double[])Api.GetPost(CoordinateTunnel.GradientId, 0))[3] * hopX;
			double b = ((double[])Api.GetPost(CoordinateTunnel.GradientId, 1))[3] * hopX;
			double c = ((double[])Api.GetPost(CoordinateTunnel.GradientId, 2))[3] * hopX;

			double a1 = ((double[])Api.GetPost(CoordinateTunnel.GradientId, 0))[4];
			double a2 = ((double[])Api.GetPost(CoordinateTunnel.GradientId, 0))[5];
			Point p1 = new Point(Convert.ToInt32(a1), Convert.ToInt32(a2));
			double b1 = ((double[])Api.GetPost(CoordinateTunnel.GradientId, 1))[4];
			double b2 = ((double[])Api.GetPost(CoordinateTunnel.GradientId, 1))[5];
			Point p2 = new Point(Convert.ToInt32(b1), Convert.ToInt32(b2));
			double c1 = ((double[])Api.GetPost(CoordinateTunnel.GradientId, 2))[4];
			double c2 = ((double[])Api.GetPost(CoordinateTunnel.GradientId, 2))[5];
			Point p3 = new Point(Convert.ToInt32(c1), Convert.ToInt32(c2));

			Point lokacija = Trilaterate(p1, a, p2, b, p3, c);

			Api.PostEntry(this.ProcessFragmentId, 0, lokacija.X);
			Api.PostEntry(this.ProcessFragmentId, 1, lokacija.Y);
		}


		/// <summary>
		/// This method is called after the PFrag queued himself for uninstall and the OS approves an uninstall.
		/// </summary>
		/// <param name="remainingTimeslots"></param>
		/// <remarks>
		/// Here you should do some pre-uninstall processing.
		/// Removing posts from the homepage is unnecessary because the OS does that for you automatically.
		/// </remarks>
		public override void Uninstall(byte remainingTimeslots)
		{
		}


		/// <summary>
		/// This method is called if the PFrag requested a transfer to the neighboring particles and
		/// his request was approved.
		/// </summary>
		/// <param name="copy">The copy of the ProcessFragmentBase.</param>
		public override void TransferGranted(ref ProcessFragmentBase copy)
		{
			CoordinateCalculator gradientCopy = (CoordinateCalculator)copy;
		}


		/// <summary>
		/// This method is called if the PFragment requested a transfer to the neighboring particles and
		/// his request was denied.
		/// </summary>
		public override void TransferRefused()
		{
		}

		/// <summary>
		/// The main processing method for Process Fragments
		/// </summary>
		/// <param name="remainingTimeslots">A value indicating the available processing time for the Update method.</param>
		/// <remarks>
		/// The time slot of 0 indicates less time available for processing and 255 more time.
		/// This is jst a suggested value, it has no bearing on the execution of the Update method (yet).
		/// </remarks>
		public override void Update(byte remainingTimeslots)
		{
			//Propagation
			// check for PaC's without Gradients and INVADE
			int mirrorCount = Api.GetIOSpaceCount();
			for (int i = 0; i < mirrorCount; i++)
			{
				// check for empty Pac's
				if (Api.GetIOPostCount(i, ProcessFragmentId) == 0)
				{
					if (!Api.IsQueuedForTransfer(i))
					{
						Api.QueueTransfer(i);
						continue;
					}
				}
			}
		}
	}

	#region Colorizer

	/// <summary>
	/// 
	/// </summary>
	public class CoordinateSeekerColorizer : HomepageColorizer
	{
		private static List<double> maximumReachedDistances = new List<double>();

		Color[] pallete = new Color[] { Color.LightGreen, Color.Red, Color.Magenta, Color.Yellow, Color.Cyan };


		/// <summary>
		/// Colorizes the specified homepage.
		/// </summary>
		/// <param name="homepage">The homepage.</param>
		/// <returns></returns>
		public override Color Colorize(Homepage homepage)
		{
			Color retval = Color.WhiteSmoke;
			for (int i = 0; i < homepage.Count; i++)
				if (homepage.KeyList[i] == CoordinateSeeker.GradientId)
				{
					retval = pallete[3];
				}


			return retval;
		}
	}

	#endregion
	[Serializable()]
	[DefaultHomepageColorizerAttribute(typeof(CoordinateSeekerColorizer))]
	[FriendlyName("Coordinate Seeker")]
	public sealed class CoordinateSeeker : ProcessFragmentBase
	{
		#region Process Fragment Identity

		public const uint GradientId = 0x2890FB4E;

		public override uint ProcessFragmentId
		{
			get { return GradientId; }
		}

		#endregion

		#region Gradient Local state

		private int dest_1 = 100;
		private int dest_2 = 100;
		private int dest_3 = 100;

		[Browsable(true)]
		[Category("1. Coordinate")]
		public int Coordinate1
		{
			get { return dest_1; }
			set { dest_1 = value; }
		}

		[Browsable(true)]
		[Category("2. Coordinate")]
		public int Coordinate2
		{
			get { return dest_2; }
			set { dest_2 = value; }
		}
		#endregion

		/// <summary>
		/// This method is called when the PFrag first enters a Paintable.
		/// </summary>
		/// <remarks>
		/// </remarks>
		public override void Install()
		{
			if (Api.GetPostCount(this.ProcessFragmentId) != 0)
			{
			}
			// everything is ok, we are allowed to install
		}


		/// <summary>
		/// This method is called after the PFrag queued himself for uninstall and the OS approves an uninstall.
		/// </summary>
		/// <param name="remainingTimeslots"></param>
		/// <remarks>
		/// Here you should do some pre-uninstall processing.
		/// Removing posts from the homepage is unnecessary because the OS does that for you automatically.
		/// </remarks>
		public override void Uninstall(byte remainingTimeslots)
		{
		}


		/// <summary>
		/// This method is called if the PFrag requested a transfer to the neighboring particles and
		/// his request was approved.
		/// </summary>
		/// <param name="copy">The copy of the ProcessFragmentBase.</param>
		public override void TransferGranted(ref ProcessFragmentBase copy)
		{
			CoordinateSeeker gradientCopy = (CoordinateSeeker)copy;

			gradientCopy.Coordinate1 = this.Coordinate1;
			gradientCopy.Coordinate2 = this.Coordinate2;
		}


		/// <summary>
		/// This method is called if the PFragment requested a transfer to the neighboring particles and
		/// his request was denied.
		/// </summary>
		public override void TransferRefused()
		{
		}

		/// <summary>
		/// The main processing method for Process Fragments
		/// </summary>
		/// <param name="remainingTimeslots">A value indicating the available processing time for the Update method.</param>
		/// <remarks>
		/// The time slot of 0 indicates less time available for processing and 255 more time.
		/// This is jst a suggested value, it has no bearing on the execution of the Update method (yet).
		/// </remarks>
		public override void Update(byte remainingTimeslots)
		{
			//Propagation
			// check for PaC's without Gradients and INVADE
			int goTo = -1;
            int min1 = Convert.ToInt32(Math.Abs(((int)Api.GetPost(CoordinateCalculator.GradientId, 0) - this.Coordinate1)));
            int min2 = Convert.ToInt32(Math.Abs(((int)Api.GetPost(CoordinateCalculator.GradientId, 1) - this.Coordinate2)));
            int tmp1;
			int tmp2;
            int mirrorCount = Api.GetIOSpaceCount();
            for (int i = 0; i < mirrorCount; i++)
			{
				// check for empty Pac's
				if (Api.GetIOPostCount(i, CoordinateCalculator.GradientId) == 2)
				{
					tmp1 = Convert.ToInt32(Math.Abs(((int)Api.GetIOPost(i, CoordinateCalculator.GradientId, 0) - this.Coordinate1)));
					tmp2 = Convert.ToInt32(Math.Abs(((int)Api.GetIOPost(i, CoordinateCalculator.GradientId, 1) - this.Coordinate2)));

					if ((tmp1 + tmp2) <= (min1 + min2))
					{
						min1 = tmp1;
						min2 = tmp2;
						if (!Api.IsQueuedForTransfer(i))
							goTo = i;
					}
				}
			}
            if (goTo > -1)
            {
                Api.QueueTransfer(goTo);
            }

		}
	}
}