/*
	Paintable Computing Simulation

	License

	This software (Paintable Computing Simulation and all parts of its 
	source code) is provided "as is", without any warranty and it is 
	the property of its respective authors. Use of this software is 
	free to everyone, except in commercial and/or military purposes 
	without written permission of its authors. Disassembling the 
	executable code of this software is forbidden for any purpose 
	without written permission of its authors.

	If you intend to use any part of the source code of this software 
	or to develop this software any further, you must include this 
	license in every source file and every About box, without changing 
	it; only appending the license is allowed. And if you use any part 
	of the source code of this software, or develop it further, please 
	send an e-mail to its authors.

	Authors of this software are not responsible for any consequences 
	which use of this software, use of any part of its source or its 
	further development may cause.

	All software and parts of source code used in the development of 
	this software are property of their respective owners.

	Good luck,
	Authors

	Authors (alphabetically):
	Fabijan Lukin
		fabijan@gmail.com
		fabijan.lukin@fer.hr
	Fran Pregernik
		fran.pregernik@gmail.com
		fran.pregernik@fer.hr
	Tomislav Sukser
		tomislav.sukser@gmail.com
		tomislav.sukser@zg.t-com.hr
		tomislav.sukser@fer.hr


	Note:
	Paintable computing was firstly introduced in Programming a 
	Paintable Computer written by William Joseph Butera.


    Copyright (c) 2005,2006. All rights reserved.
    4-6/2006

 */
using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using PaintableSimulator.Engine;
using PaintableSimulator.Engine.PaintableComputers.OperatingSystem;
using PaintableSimulator.Engine.PaintableComputers;

namespace PaintableSimulator.Programs
{
    public sealed class BojanjeVirusa : HomepageColorizer
    {

        public override System.Drawing.Color Colorize(Homepage homepage)
        {
            // Prolazimo kroz sve zapise na podatkovnoj stranici s
            // iteratorom i. 
            for (int i = 0; i < homepage.Count; i++)
            {
                // Ukoliko naemo zapis koji odgovara programu koji ima
                // identifikator kao i MojVirus (0x1369), ...
                if (homepage.KeyList[i] == 0x1369)
                {
                    // ... te ako na tom dijelu podatkovne stranice postoje zapisi
                    if (homepage.DataList[i].Count > 0)
                    {
                        // obojat emo raunalo u crveno
                        return Color.Red;
                    }
                }
            }

            // U protivnom, obojat emo raunalo u zeleno.
            return Color.Green;
        }
    }


    [DefaultHomepageColorizerAttribute(typeof(BojanjeVirusa))]
    [FriendlyDescription("Ovo je svjee napisani program")]
    [FriendlyName("MojVirus")]
    [Serializable()]
    public sealed class MojVirus : ProcessFragmentBase
    {
        public override uint ProcessFragmentId
        {
            get { return 0x1369; }
        }

        public override void Install()
        {
            // Ako postoje naznake da je virus ve ovdje...
            if (Api.GetPostCount() > 0)
            {
                // ...deinstalirat emo ga...
                Api.QueueUninstall();
                return;
            }
            else
            {
                // ...a u protivnom postavit emo na zapis nultog indeksa
                // na stranicu predvienu za ovaj program oznaku da smo 
                // ovdje.
                Api.PostEntry(0, "MojVirus");
            }
        }

        public override void Uninstall(byte remainingTimeslots)
        {
            // Nita.
        }

        public override void TransferGranted(ref ProcessFragmentBase copy)
        {
            // Nita.
        }

        public override void TransferRefused()
        {
            // Nita.
        }

        public override void Update(byte remainingTimeslots)
        {
            // Ii emo iteratorom i po svim susjedima
            for (int i = 0; i < Api.GetIOSpaceCount(); i++)
            {
                // i ako doemo do nekog (i-tog) koji nema oznaku
                // da je zaraen virusom, zahtijevat emo
                // kopiranje na njega (na i-tog susjeda).
                if (Api.GetIOPostCount(i) <= 0)
                {
                    Api.QueueTransfer(i);
                }
            }
        }
    }
}
