/*
	Paintable Computing Simulation

	License

	This software (Paintable Computing Simulation and all parts of its 
	source code) is provided "as is", without any warranty and it is 
	the property of its respective authors. Use of this software is 
	free to everyone, except in commercial and/or military purposes 
	without written permission of its authors. Disassembling the 
	executable code of this software is forbidden for any purpose 
	without written permission of its authors.

	If you intend to use any part of the source code of this software 
	or to develop this software any further, you must include this 
	license in every source file and every About box, without changing 
	it; only appending the license is allowed. And if you use any part 
	of the source code of this software, or develop it further, please 
	send an e-mail to its authors.

	Authors of this software are not responsible for any consequences 
	which use of this software, use of any part of its source or its 
	further development may cause.

	All software and parts of source code used in the development of 
	this software are property of their respective owners.

	Good luck,
	Authors

	Authors (alphabetically):
	Fabijan Lukin
		fabijan@gmail.com
		fabijan.lukin@fer.hr
	Fran Pregernik
		fran.pregernik@gmail.com
		fran.pregernik@fer.hr
	Tomislav Sukser
		tomislav.sukser@gmail.com
		tomislav.sukser@zg.t-com.hr
		tomislav.sukser@fer.hr


	Note:
	Paintable computing was firstly introduced in Programming a 
	Paintable Computer written by William Joseph Butera.


    Copyright (c) 2005,2006. All rights reserved.
    4-6/2006

 */
#region Using Directives

using System;
using System.ComponentModel;
using System.Drawing;
using PaintableSimulator.Engine;
using PaintableSimulator.Engine.PaintableComputers.OperatingSystem;
using PaintableSimulator.Engine.PaintableComputers;

#endregion

namespace PaintableSimulator.Programs
{

	#region Colorizer

	/// <summary>
	/// 
	/// </summary>
	public class PulsatorColorizer : HomepageColorizer
	{
		/// <summary>
		/// Colorizes the specified homepage.
		/// </summary>
		/// <param name="homepage">The homepage.</param>
		/// <returns></returns>
		public override Color Colorize(Homepage homepage)
		{
			for (int i = 0; i < homepage.Count; i++)
				if (homepage.KeyList[i] == 0x1EB7353e)
					if (homepage.DataList[i].Count > 0)
					{
						return (Color)homepage.DataList[i][4];
					}

			return Color.WhiteSmoke;
		}
	}

	#endregion

	[FriendlyDescription("Pulsing around")]
	[FriendlyName("Pulsator")]
	[DefaultHomepageColorizerAttribute(typeof(PulsatorColorizer))]
	[Serializable()]
	public sealed class Pulsator : ProcessFragmentBase
	{
		#region Process Fragment Identity

		public override uint ProcessFragmentId
		{
			get { return 0x1EB7353e; }
		}

		#endregion

		#region Pulsator Local state

		private int pulsAfter = 25;

		[Browsable(true)]
		[Category("Pulsing frequency")]
		public int PulsAfter
		{
			get { return pulsAfter; }
			set
			{
				if (value > 0)
					pulsAfter = value;
				else
					pulsAfter = 10;
			}
		}

		private int maxHopCount = 200;

		[Browsable(true)]
		[Category("Maximal Radius in Hops")]
		public int MaxHopCount
		{
			get { return maxHopCount; }
			set
			{
				if (value > 1)
					maxHopCount = value;
				else
					maxHopCount = 1;
			}
		}

		private int propegationDelay = 5;

		[Browsable(true)]
		[Category("Color propegation delay")]
		public int PropegationDelay
		{
			get { return propegationDelay; }
			set
			{
				if (value > 1)
					propegationDelay = value;
				else
					propegationDelay = 1;
			}
		}

		private bool listener;
		private int updateCount = 0;
		private int pulsCount = 0;

		#endregion

		/// <summary>
		/// This method is called when the PFrag first enters a Paintable.
		/// </summary>
		/// <param name="initialState">The initial state vector passed on by his ancestor.</param>
		/// <remarks>
		/// </remarks>
		public override void Install()
		{
			if (Api.GetPostCount(this.ProcessFragmentId) == 0)
			{
				updateCount = 0;
				// everything is ok, we are allowed to install
				if (!listener)
				{
					Api.PostEntry(ProcessFragmentId, 0, "Pulsator");
					Api.PostEntry(ProcessFragmentId, 1, this.PulsAfter);
					Api.PostEntry(this.ProcessFragmentId, 3, 0);
					Api.PostEntry(this.ProcessFragmentId, 4, Color.Aqua);
					Api.PostEntry(this.ProcessFragmentId, 7, 0);
					Api.PostEntry(this.ProcessFragmentId, 8, Color.Aqua);

					Api.PostEntry(this.ProcessFragmentId, 5, this.propegationDelay);
					Api.PostEntry(this.ProcessFragmentId, 6, this.propegationDelay);
				}
				else
				{
					Api.PostEntry(ProcessFragmentId, 0, "PulsListener");
					Api.PostEntry(ProcessFragmentId, 1, 0);
					Api.PostEntry(this.ProcessFragmentId, 3, 0);
					Api.PostEntry(this.ProcessFragmentId, 4, Color.BlanchedAlmond);
					Api.PostEntry(this.ProcessFragmentId, 7, 0);
					Api.PostEntry(this.ProcessFragmentId, 8, Color.BlanchedAlmond);

					Api.PostEntry(this.ProcessFragmentId, 5, this.propegationDelay);
					Api.PostEntry(this.ProcessFragmentId, 6, this.propegationDelay);
				}

				updateCount = 0;
			}
			else
			{
				Api.QueueUninstall();
			}
		}

		/// <summary>
		/// This method is called after the PFrag queued himself for uninstall and the OS approves an uninstall.
		/// </summary>
		/// <param name="remainingTimeslots"></param>
		/// <remarks>
		/// Here you sould do some pre uninstall processing.
		/// Removing posts from the homepage is unnecessary because the os does that for you automatically.
		/// </remarks>
		public override void Uninstall(byte remainingTimeslots)
		{
		}

		/// <summary>
		/// This method is called if the PFrag requested a transfer to the neighboring particles and
		/// his request was approved.
		/// </summary>
		/// <param name="copy">The copy of the ProcessFragmentBase.</param>
		/// <remarks>
		/// In this method you should set saveState to a copy of your own ProcessFragmentState with modified values.
		/// </remarks>
		public override void TransferGranted(ref ProcessFragmentBase copy)
		{
			((Pulsator)copy).listener = true;
			((Pulsator)copy).PropegationDelay = this.propegationDelay;
			((Pulsator)copy).maxHopCount = this.maxHopCount + 1;
		}

		/// <summary>
		/// This method is called if the PFragment requested a transfer to the neighboring particles and
		/// his request was denied.
		/// </summary>
		public override void TransferRefused()
		{
		}

		/// <summary>
		/// The main processing method for Process Fragments
		/// </summary>
		/// <param name="remainingTimeslots">A value indicating the available processing time for the Update method.</param>
		/// <remarks>
		/// The time slot of 0 indicates less time available for processing and 255 more time.
		/// This is just a suggested value, it has no bearing on the execution of the Update method (yet).
		/// </remarks>
		public override void Update(byte remainingTimeslots)
		{
			// Sirenje
			int mirrorCount = Api.GetIOSpaceCount();
			for (int i = 0; i < mirrorCount; i++)
			{
				int tmp = Api.GetIOPostCount(i, ProcessFragmentId);
				if (tmp == 0)
				{
					Api.QueueTransfer(i);
				}
			}

			if (Api.IsQueuedForTransfer())
				return;


			Object tmpObject;
			tmpObject = Api.GetPost(this.ProcessFragmentId, 0);
			if (tmpObject == null)
				return;

			//Initiator
			if (((string)tmpObject) == "Pulsator")
			{
				updateCount++;
				int pulsAfter = (int)Api.GetPost(this.ProcessFragmentId, 1);
				if (updateCount > pulsAfter)
				{
					updateCount = 0;

					pulsCount++;

					Api.PostEntry(this.ProcessFragmentId, 3, pulsCount);
					Color c;
					if ((pulsCount % 2) == 0)
						c = Color.Blue;
					else
						c = Color.White;
					Api.PostEntry(this.ProcessFragmentId, 4, c);
				}
			}
			//Just change color and propagate color message
			else
			{
				tmpObject = Api.GetPost(this.ProcessFragmentId, 7);
				int thisPCPulsCount = 0;
				if (tmpObject != null)
					thisPCPulsCount = (int)tmpObject;

				tmpObject = Api.GetPost(this.ProcessFragmentId, 4);
				Color thisPCColor = Color.Black;
				if (tmpObject != null)
					thisPCColor = (Color)tmpObject;

				int maxPuls = -1;
				Color maxColor = Color.Black;

				mirrorCount = Api.GetIOSpaceCount();
				for (int i = 0; i < mirrorCount; i++)
				{
					tmpObject = Api.GetIOPost(i, this.ProcessFragmentId, 3);
					if (tmpObject == null)
						continue;
					if (((int)tmpObject) > thisPCPulsCount)
					{
						maxPuls = (int)tmpObject;
						maxColor = (Color)Api.GetIOPost(i, this.ProcessFragmentId, 4);
					}
				}

				if (maxPuls > thisPCPulsCount)
				{
					tmpObject = Api.GetPost(5);

					Api.PostEntry(this.ProcessFragmentId, 7, maxPuls);
					Api.PostEntry(this.ProcessFragmentId, 8, maxColor);
					Api.PostEntry(this.ProcessFragmentId, 6, tmpObject);
				}
				else
				{
					tmpObject = Api.GetPost(6);
					Api.PostEntry(this.ProcessFragmentId, 6, (int)tmpObject - 1);
					if (((int)tmpObject) == 0)
					{
						Api.PostEntry(this.ProcessFragmentId, 3, Api.GetPost(7));
						Api.PostEntry(this.ProcessFragmentId, 4, Api.GetPost(8));

					}
				}
			}
		}
	}
}