/*
	Paintable Computing Simulation

	License

	This software (Paintable Computing Simulation and all parts of its 
	source code) is provided "as is", without any warranty and it is 
	the property of its respective authors. Use of this software is 
	free to everyone, except in commercial and/or military purposes 
	without written permission of its authors. Disassembling the 
	executable code of this software is forbidden for any purpose 
	without written permission of its authors.

	If you intend to use any part of the source code of this software 
	or to develop this software any further, you must include this 
	license in every source file and every About box, without changing 
	it; only appending the license is allowed. And if you use any part 
	of the source code of this software, or develop it further, please 
	send an e-mail to its authors.

	Authors of this software are not responsible for any consequences 
	which use of this software, use of any part of its source or its 
	further development may cause.

	All software and parts of source code used in the development of 
	this software are property of their respective owners.

	Good luck,
	Authors

	Authors (alphabetically):
	Fabijan Lukin
		fabijan@gmail.com
		fabijan.lukin@fer.hr
	Fran Pregernik
		fran.pregernik@gmail.com
		fran.pregernik@fer.hr
	Tomislav Sukser
		tomislav.sukser@gmail.com
		tomislav.sukser@zg.t-com.hr
		tomislav.sukser@fer.hr


	Note:
	Paintable computing was firstly introduced in Programming a 
	Paintable Computer written by William Joseph Butera.


    Copyright (c) 2005,2006. All rights reserved.
    4-6/2006

 */

using System;
using System.IO;
using System.Net;
using System.Net.NetworkInformation;
using System.Reflection;
using System.Runtime.Remoting;
using System.Runtime.Remoting.Channels;
using System.Runtime.Remoting.Channels.Tcp;
using System.Threading;
using System.Windows.Forms;
using PaintableSimulator.Engine;
using System.Diagnostics;
using System.Configuration;
using System.Net.Sockets;
using PaintableSimulator.Engine.PaintableComputers.Sensors;
using System.Runtime.Remoting.Channels.Http;
using PaintableSimulator.SmartClient.Resources;
using PaintableSimulator.ServerInterface;

namespace PaintableSimulator.SmartClient
{
	internal static class Program
	{
		public static string ClientId = RandomNumberGenerator.GetRandomNumber.ToString("X");
		public static ServerInterfaceBase Factory;
		public static RemoteSimulator simulator;
		public static bool EnableRemoting = false;
		public static IPAddress ServerIP = new IPAddress(new byte[] { 192, 168, 1, 103 });
		public static uint ServerPort = 8989;
		public static string ServerApplicationName = "PaintableSimulatorServer";

		private static bool handlingIrrecoverableError = false;

		private static DirectoryInfo pluginsDirectory;

		// Create a logger for use in this class
		internal static readonly log4net.ILog log = log4net.LogManager.GetLogger("GUI");
		// NOTE that using System.Reflection.MethodBase.GetCurrentMethod().DeclaringType
		// is equivalent to typeof(LoggingExample) but is more portable
		// i.e. you can copy the code directly into another class without
		// needing to edit the code.

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		private static void Main(string[] args)
		{

			// select the data/UI language
			System.Threading.Thread.CurrentThread.CurrentCulture = System.Globalization.CultureInfo.InstalledUICulture;
			System.Threading.Thread.CurrentThread.CurrentUICulture = System.Globalization.CultureInfo.InstalledUICulture;

			// check the plugins folder
			FileInfo assemblyFileInfo = new FileInfo(Assembly.GetExecutingAssembly().Location);
			DirectoryInfo assemblyDirectory = assemblyFileInfo.Directory;
			pluginsDirectory = new DirectoryInfo(assemblyDirectory.FullName + "\\Plugins");
			if (!pluginsDirectory.Exists)
				pluginsDirectory.Create();

			// misc
			Application.EnableVisualStyles();
			Application.SetCompatibleTextRenderingDefault(false);
			Application.SetUnhandledExceptionMode(UnhandledExceptionMode.CatchException, true);
			Application.ApplicationExit += new EventHandler(Application_ApplicationExit);
			Application.ThreadException += new ThreadExceptionEventHandler(Application_ThreadException);
			AppDomain.CurrentDomain.AssemblyResolve += new ResolveEventHandler(CurrentDomain_AssemblyResolve);
			AppDomain.CurrentDomain.UnhandledException += new UnhandledExceptionEventHandler(CurrentDomain_UnhandledException);


#if DEBUG
			// configure log4net with the DEBUG treshold
			Stream s = Assembly.GetExecutingAssembly().GetManifestResourceStream("PaintableSimulator.SmartClient.log4net.DebugLogConfig.xml");
#else
			// configure log4net with the WARN treshold
			Stream s = Assembly.GetExecutingAssembly().GetManifestResourceStream("PaintableSimulator.SmartClient.log4net.ReleaseLogConfig.xml");
#endif
			log4net.Config.XmlConfigurator.Configure(s);


			// check if we are running a remote simulator
			for (int i = 0; i < args.Length; i++)
				if (args[i] == "/remote")
				{
					EnableRemoting = true;

					// show the remoting startup wizard
					StartupWizard sw = new StartupWizard();
					if (sw.ShowDialog() == DialogResult.Cancel)
					{
						// exit application
						return;
					}

					ChannelServices.RegisterChannel(new TcpChannel(0), false);

					string tcpAddress =
						string.Format("tcp://{0}:{1}/{2}", ServerIP.ToString(), ServerPort.ToString(), ServerApplicationName);

					// create a factory
					try
					{
						Factory = (ServerInterfaceBase)Activator.GetObject(typeof(ServerInterfaceBase), tcpAddress);
					}
					catch (Exception ex)
					{
						MessageBox.Show(UserMessages.RemotingConfigurationError);
						if (log.IsErrorEnabled)
							log.ErrorFormat(UserMessages.RemotingConfigurationErrorLog, ex);

						return;
					}

					if (log.IsInfoEnabled)
						log.Info(UserMessages.RemotingConfigurationSuccess);



				}



			// start main form
			Application.Run(new SimulatorClient());
		}

		/// <summary>
		/// Handles any irrecoverable remoting exceptions.
		/// </summary>
		public static void HandleIrrecoverableRemotingException(Exception ex)
		{

			if (handlingIrrecoverableError)
				return;

			handlingIrrecoverableError = true;
			
			if (log.IsFatalEnabled)
				log.FatalFormat(UserMessages.FatalRemotingError, ex.ToString());

			// irrecoverable exception
			DialogResult result =
				MessageBox.Show(UserMessages.FatalRemotingMessage, "", MessageBoxButtons.YesNo);

			if (result == DialogResult.Yes)
			{

				try
				{
					Application.ExitThread();
					AppDomain.Unload(AppDomain.CurrentDomain);
					Application.DoEvents();
					Thread.CurrentThread.Abort();
					Application.Exit();
				}
				catch
				{ }
				
			}
			else
			{
				Application.Restart();
			}
		}


		/// <summary>
		/// Handles the AssemblyResolve event of the CurrentDomain control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="args">The <see cref="T:System.ResolveEventArgs"/> instance containing the event data.</param>
		/// <returns></returns>
		private static Assembly CurrentDomain_AssemblyResolve(object sender, ResolveEventArgs args)
		{
			// get the dlls in the directory
			FileInfo[] dlls = pluginsDirectory.GetFiles("*.dll");

			// log search attempt
			if (log.IsInfoEnabled)
				log.InfoFormat(UserMessages.AssemblySearching, args.Name);

			// load assemblies
			try
			{

				for (int i = 0; i < dlls.Length; i++)
				{
					if (Assembly.ReflectionOnlyLoadFrom(dlls[i].FullName).FullName == args.Name)
						return Assembly.LoadFile(dlls[i].FullName);
				}
			}
			catch (Exception ex)
			{
				// log loading failed
				if (log.IsErrorEnabled)
					log.ErrorFormat(UserMessages.AssemblyLoadingFailed, ex);
			}

			// log assembly not found
			if (log.IsWarnEnabled)
				log.Warn(UserMessages.AssemblyNotFound);


			return null;
		}

		/// <summary>
		/// Handles the UnhandledException event of the CurrentDomain control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="e">The <see cref="T:System.UnhandledExceptionEventArgs"/> instance containing the event data.</param>
		private static void CurrentDomain_UnhandledException(object sender, UnhandledExceptionEventArgs e)
		{
			if (log.IsErrorEnabled)
				log.ErrorFormat(UserMessages.UnhandledDomainException, (Exception)e.ExceptionObject);
		}

		/// <summary>
		/// Handles the ThreadException event of the Application control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="e">The <see cref="T:System.Threading.ThreadExceptionEventArgs"/> instance containing the event data.</param>
		private static void Application_ThreadException(object sender, ThreadExceptionEventArgs e)
		{
			if (log.IsErrorEnabled)
				log.ErrorFormat(UserMessages.UnhandledThreadException, e.Exception);
		}


		/// <summary>
		/// Handles the ApplicationExit event of the Application control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="e">The <see cref="T:System.EventArgs"/> instance containing the event data.</param>
		private static void Application_ApplicationExit(object sender, EventArgs e)
		{
			if (simulator != null)
			{

				try
				{
					simulator.Terminate();
					Application.DoEvents();
					System.Threading.Thread.Sleep(1000);
				}
				catch (RemotingException ex)
				{
					HandleIrrecoverableRemotingException(ex);
				}
				catch (SocketException ex)
				{
					HandleIrrecoverableRemotingException(ex);
				}
				finally
				{
					simulator = null;
				}
			}


		}
	}
}