/*
	Paintable Computing Simulation

	License

	This software (Paintable Computing Simulation and all parts of its 
	source code) is provided "as is", without any warranty and it is 
	the property of its respective authors. Use of this software is 
	free to everyone, except in commercial and/or military purposes 
	without written permission of its authors. Disassembling the 
	executable code of this software is forbidden for any purpose 
	without written permission of its authors.

	If you intend to use any part of the source code of this software 
	or to develop this software any further, you must include this 
	license in every source file and every About box, without changing 
	it; only appending the license is allowed. And if you use any part 
	of the source code of this software, or develop it further, please 
	send an e-mail to its authors.

	Authors of this software are not responsible for any consequences 
	which use of this software, use of any part of its source or its 
	further development may cause.

	All software and parts of source code used in the development of 
	this software are property of their respective owners.

	Good luck,
	Authors

	Authors (alphabetically):
	Fabijan Lukin
		fabijan@gmail.com
		fabijan.lukin@fer.hr
	Fran Pregernik
		fran.pregernik@gmail.com
		fran.pregernik@fer.hr
	Tomislav Sukser
		tomislav.sukser@gmail.com
		tomislav.sukser@zg.t-com.hr
		tomislav.sukser@fer.hr


	Note:
	Paintable computing was firstly introduced in Programming a 
	Paintable Computer written by William Joseph Butera.


    Copyright (c) 2005,2006. All rights reserved.
    4-6/2006

 */
using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using System.Windows.Forms;

namespace PaintableSimulator.SmartClient
{
	partial class SimulatorClient
	{
		/// <summary>
		/// 
		/// </summary>
		private enum SelectorType
		{
			Round,
			Square,
			Polygon
		}

		bool selectorEnabled = false;
		SelectorType selectorType = SelectorType.Round;
		Point selectorCurrentPoint = Point.Empty;

		/// <summary>
		/// Initializes the delete selectors.
		/// </summary>
		private void InitializeDeleteSelectors()
		{
			// round selector events
			roundSelectorMouseWheel = new MouseEventHandler(this.RoundSelector_MouseWheelEvent);
			roundSelectorMouseMove = new MouseEventHandler(this.RoundSelector_MouseEvent);
			roundSelectorMouseDown = new MouseEventHandler(this.RoundSelector_MouseDownEvent);
			roundSelectorMouseUp = new MouseEventHandler(this.RoundSelector_MouseUpEvent);
			roundSelectorKeyDown = new KeyEventHandler(this.RoundSelector_KeyDownEvent);

			// square selector events
			squareSelectorMouseMove = new MouseEventHandler(this.SquareSelector_MouseMoveEvent);
			squareSelectorMouseDown = new MouseEventHandler(this.SquareSelector_MouseDownEvent);
			squareSelectorMouseUp = new MouseEventHandler(this.SquareSelector_MouseUpEvent);
			squareSelectorKeyDown = new KeyEventHandler(this.SquareSelector_KeyDownEvent);

			// polygon selectro events
			polygonSelectorKeyDown = new KeyEventHandler(this.PolygonSelector_KeyDownEvent);
			polygonSelectorMouseDown = new MouseEventHandler(this.PolygonSelector_MouseDownEvent);
		}

		/// <summary>
		/// Draws the selector.
		/// </summary>
		/// <param name="graphics">The graphics.</param>
		private void DrawSelector(Graphics graphics)
		{
			if (!selectorEnabled)
				return;

			switch (selectorType)
			{
				case SelectorType.Round:
					{
						SolidBrush fill = new SolidBrush(Color.FromArgb(70, 0, 0, 255));

						for (int i = 0; i < roundSelectorCenterPoints.Count; i++)
						{
							// draw round selector range
							graphics.FillEllipse(fill, CreateCircleAroundPoint(roundSelectorCenterPoints[i], roundSelectorRadius));
						}

						// draw round selector range
						graphics.FillEllipse(fill, CreateCircleAroundPoint(selectorCurrentPoint, roundSelectorRadius));
						graphics.DrawEllipse(Pens.Blue, CreateCircleAroundPoint(selectorCurrentPoint, roundSelectorRadius));

						break;
					}

				case SelectorType.Polygon:
					{
						if (polygonSelectorPoints.Count == 0)
							return;

						// draw rectangle selector
						Point[] points = polygonSelectorPoints.ToArray();
						SolidBrush fill = new SolidBrush(Color.FromArgb(70, 0, 0, 255));

						graphics.FillPolygon(fill, points);
						graphics.DrawPolygon(Pens.Blue, points);


						break;
					}

				case SelectorType.Square:
					{
						if (!squareSelectorDragging)
							return;

						// draw rectangle selector
						SolidBrush fill = new SolidBrush(Color.FromArgb(70, 0, 0, 255));

						Point startPoint = Point.Empty;
						Size size = Size.Empty;

						if (squareSelectorStartPoint.X > selectorCurrentPoint.X)
						{
							startPoint.X = selectorCurrentPoint.X;
							size.Width = squareSelectorStartPoint.X - selectorCurrentPoint.X;
						}
						else
						{
							startPoint.X = squareSelectorStartPoint.X;
							size.Width = selectorCurrentPoint.X - squareSelectorStartPoint.X;
						}

						if (squareSelectorStartPoint.Y > selectorCurrentPoint.Y)
						{
							startPoint.Y = selectorCurrentPoint.Y;
							size.Height = squareSelectorStartPoint.Y - selectorCurrentPoint.Y;
						}
						else
						{
							startPoint.Y = squareSelectorStartPoint.Y;
							size.Height = selectorCurrentPoint.Y - squareSelectorStartPoint.Y;

						}

						graphics.FillRectangle(fill, new Rectangle(startPoint, size));
						graphics.DrawRectangle(Pens.Blue, new Rectangle(startPoint, size));

						break;
					}
				default:
					break;
			}
		}

		#region Round Selector

		uint roundSelectorRadius = 20;
		bool roundSelectorContinuous = false;
		List<int> roundSelectorIndices = new List<int>();
		List<Point> roundSelectorCenterPoints = new List<Point>();

		MouseEventHandler roundSelectorMouseWheel;
		MouseEventHandler roundSelectorMouseMove;
		MouseEventHandler roundSelectorMouseDown;
		MouseEventHandler roundSelectorMouseUp;
		KeyEventHandler roundSelectorKeyDown;


		/// <summary>
		/// Enables the selector.
		/// </summary>
		private void EnableRoundSelector()
		{
			// disable any debuggers
			DisableDebugger();

			simulatorDisplay.Cursor = System.Windows.Forms.Cursors.Cross;

			// disable other selectors
			DisablePolygonSelector();
			DisableSquareSelector();

			selectorCurrentPoint = Point.Empty;

			roundSelectorIndices.Clear();
			roundSelectorCenterPoints.Clear();

			selectorType = SelectorType.Round;

			simulatorDisplay.MouseWheel += roundSelectorMouseWheel;
			simulatorDisplay.MouseUp += roundSelectorMouseUp;
			simulatorDisplay.MouseDown += roundSelectorMouseDown;
			simulatorDisplay.MouseMove += roundSelectorMouseMove;
			this.KeyDown += roundSelectorKeyDown;

			selectorEnabled = true;
		}

		/// <summary>
		/// Disables the round selector.
		/// </summary>
		private void DisableRoundSelector()
		{
			selectorEnabled = false;

			simulatorDisplay.MouseWheel -= roundSelectorMouseWheel;
			simulatorDisplay.MouseUp -= roundSelectorMouseUp;
			simulatorDisplay.MouseDown -= roundSelectorMouseDown;
			simulatorDisplay.MouseMove -= roundSelectorMouseMove;
			this.KeyDown -= roundSelectorKeyDown;

			simulatorDisplay.Cursor = System.Windows.Forms.Cursors.Default;
		}


		/// <summary>
		/// Handles the KeyDown event of the SimulatorClient control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="e">The <see cref="T:System.Windows.Forms.KeyEventArgs"/> instance containing the event data.</param>
		private void RoundSelector_KeyDownEvent(object sender, KeyEventArgs e)
		{
			if (e.KeyCode == Keys.Escape)
			{
				DisableRoundSelector();
				selectorEnabled = false;
				e.Handled = true;
			}

			if (e.Control && (!roundSelectorContinuous))
			{
				if (e.KeyCode == Keys.Up)
				{
					roundSelectorRadius += 5;
					e.Handled = true;
				}
				else if (e.KeyCode == Keys.Down)
				{
					roundSelectorRadius -= 5;
					e.Handled = true;
				}

				// check size limits
				if (roundSelectorRadius > 100)
					roundSelectorRadius = 99;

				if (roundSelectorRadius < 10)
					roundSelectorRadius = 10;


			}
		}


		/// <summary>
		/// Handles the MouseWheelEvent event of the RoundSelector control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="e">The <see cref="T:System.Windows.Forms.MouseEventArgs"/> instance containing the event data.</param>
		void RoundSelector_MouseWheelEvent(object sender, System.Windows.Forms.MouseEventArgs e)
		{

			if (!roundSelectorContinuous)
			{
				// resize the radius with the mouse wheel
				roundSelectorRadius += (uint)(e.Delta / 4);

				// check size limits
				if (roundSelectorRadius > 100)
					roundSelectorRadius = 99;

				if (roundSelectorRadius < 10)
					roundSelectorRadius = 10;
			}

		}

		/// <summary>
		/// Handles the MouseEvent event of the RoundSelector control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="e">The <see cref="T:System.Windows.Forms.MouseEventArgs"/> instance containing the event data.</param>
		void RoundSelector_MouseEvent(object sender, System.Windows.Forms.MouseEventArgs e)
		{

			if (!roundSelectorContinuous)
			{
				// resize the radius with the mouse wheel
				roundSelectorRadius += (uint)(e.Delta / 4);

				// check size limits
				if (roundSelectorRadius > 100)
					roundSelectorRadius = 99;

				if (roundSelectorRadius < 10)
					roundSelectorRadius = 10;
			}


			// set the selector center point to the current mouse location
			selectorCurrentPoint = TransformCoordinatePageToWorld(e.Location);

			if (roundSelectorContinuous)
			{
				AddRoundSelectorCenterPoint(selectorCurrentPoint);
			}

		}

		/// <summary>
		/// Handles the MouseDownEvent event of the RoundSelector control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="e">The <see cref="T:System.Windows.Forms.MouseEventArgs"/> instance containing the event data.</param>
		void RoundSelector_MouseDownEvent(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			roundSelectorContinuous = true;

			// add the last center point
			AddRoundSelectorCenterPoint(TransformCoordinatePageToWorld(e.Location));

		}


		/// <summary>
		/// Handles the MouseUpEvent event of the RoundSelector control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="e">The <see cref="T:System.Windows.Forms.MouseEventArgs"/> instance containing the event data.</param>
		void RoundSelector_MouseUpEvent(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			roundSelectorContinuous = false;

			// disable selector
			DisableRoundSelector();

			// add the last center point
			AddRoundSelectorCenterPoint(TransformCoordinatePageToWorld(e.Location));



			lock (displaySyncRoot)
			{

				// disable selected computers
				Program.simulator.DisableComputers(roundSelectorIndices, true);

				// refresh the lists
				if (Program.EnableRemoting)
					completeComputerDataList = Program.simulator.ComputerList;


			}

		}




		/// <summary>
		/// Adds the round selector center point.
		/// </summary>
		/// <param name="center">The center.</param>
		private void AddRoundSelectorCenterPoint(Point center)
		{

			if (!roundSelectorCenterPoints.Contains(center))
			{
				roundSelectorCenterPoints.Add(center);

				// delete computers
				for (int i = 0; i < completeComputerDataList.Count; i++)
				{
					Point pcLocation = completeComputerDataList[i].Location;
					double d = Math.Sqrt(Math.Pow((pcLocation.X - center.X), 2) + Math.Pow((pcLocation.Y - center.Y), 2));

					if (((int)d <= roundSelectorRadius) && (!roundSelectorIndices.Contains(i)))
						roundSelectorIndices.Add(i);

				}

			}

		}



		#endregion

		#region Square selector

		private Point squareSelectorStartPoint = Point.Empty;
		private bool squareSelectorDragging = false;

		MouseEventHandler squareSelectorMouseMove;
		MouseEventHandler squareSelectorMouseDown;
		MouseEventHandler squareSelectorMouseUp;
		KeyEventHandler squareSelectorKeyDown;


		/// <summary>
		/// Enables the square computer selector.
		/// </summary>
		private void EnableSquareSelector()
		{
			// disable any debuggers
			DisableDebugger();


			simulatorDisplay.Cursor = System.Windows.Forms.Cursors.Cross;

			// disable other selectors
			DisablePolygonSelector();
			DisableRoundSelector();

			selectorCurrentPoint = Point.Empty;


			selectorType = SelectorType.Square;

			simulatorDisplay.MouseDown += squareSelectorMouseDown;
			simulatorDisplay.MouseUp += squareSelectorMouseUp;
			simulatorDisplay.MouseMove += squareSelectorMouseMove;
			this.KeyDown += squareSelectorKeyDown;

			selectorEnabled = true;
		}


		/// <summary>
		/// Disables the square selector.
		/// </summary>
		private void DisableSquareSelector()
		{
			selectorEnabled = true;

			simulatorDisplay.MouseDown -= squareSelectorMouseDown;
			simulatorDisplay.MouseUp -= squareSelectorMouseUp;
			simulatorDisplay.MouseMove -= squareSelectorMouseMove;
			this.KeyDown -= squareSelectorKeyDown;

			squareSelectorDragging = false;

			simulatorDisplay.Cursor = System.Windows.Forms.Cursors.Default;

		}


		/// <summary>
		/// Handles the MouseDownEvent event of the SquareSelector control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="e">The <see cref="T:System.Windows.Forms.MouseEventArgs"/> instance containing the event data.</param>
		void SquareSelector_MouseDownEvent(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			if (!squareSelectorDragging)
			{
				squareSelectorStartPoint = TransformCoordinatePageToWorld(e.Location);
				selectorCurrentPoint = TransformCoordinatePageToWorld(e.Location);
			}

			squareSelectorDragging = true;
		}

		/// <summary>
		/// Handles the MouseUpEvent event of the SquareSelector control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="e">The <see cref="T:System.Windows.Forms.MouseEventArgs"/> instance containing the event data.</param>
		void SquareSelector_MouseUpEvent(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			squareSelectorDragging = false;

			DisableSquareSelector();


			lock (displaySyncRoot)
			{
				// delete computers
				List<int> indices = new List<int>();

				for (int i = 0; i < completeComputerDataList.Count; i++)
				{
					Point pcLocation = completeComputerDataList[i].Location;

					bool conditionX = ((squareSelectorStartPoint.X > selectorCurrentPoint.X) &&
									(selectorCurrentPoint.X < pcLocation.X) &&
									(pcLocation.X < squareSelectorStartPoint.X))
									||
									((squareSelectorStartPoint.X < selectorCurrentPoint.X) &&
									(squareSelectorStartPoint.X < pcLocation.X) &&
									(pcLocation.X < selectorCurrentPoint.X));

					bool conditionY = ((squareSelectorStartPoint.Y > selectorCurrentPoint.Y) &&
									(selectorCurrentPoint.Y < pcLocation.Y) &&
									(pcLocation.Y < squareSelectorStartPoint.Y))
									||
									((squareSelectorStartPoint.Y < selectorCurrentPoint.Y) &&
									(squareSelectorStartPoint.Y < pcLocation.Y) &&
									(pcLocation.Y < selectorCurrentPoint.Y));

					if (conditionX && conditionY)
						indices.Add(i);

				}

				// disable the computers
				Program.simulator.DisableComputers(indices, true);

				// refresh the lists
				if (Program.EnableRemoting)
					completeComputerDataList = Program.simulator.ComputerList;


			}

		}

		/// <summary>
		/// Handles the MouseMoveEvent event of the SquareSelector control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="e">The <see cref="T:System.Windows.Forms.MouseEventArgs"/> instance containing the event data.</param>
		void SquareSelector_MouseMoveEvent(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			if (squareSelectorDragging)
				selectorCurrentPoint = TransformCoordinatePageToWorld(e.Location);
		}


		/// <summary>
		/// Handles the KeyDown event of the SquareSelector control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="e">The <see cref="T:System.Windows.Forms.KeyEventArgs"/> instance containing the event data.</param>
		private void SquareSelector_KeyDownEvent(object sender, KeyEventArgs e)
		{
			if (e.KeyCode == Keys.Escape)
			{
				DisableSquareSelector();
				selectorEnabled = false;
				e.Handled = true;
			}
		}

		#endregion

		#region Polygon selector

		private List<Point> polygonSelectorPoints = new List<Point>();

		MouseEventHandler polygonSelectorMouseDown;
		KeyEventHandler polygonSelectorKeyDown;

		/// <summary>
		/// Enables the Polygon computer selector.
		/// </summary>
		private void EnablePolygonSelector()
		{
			// disable any debuggers
			DisableDebugger();

			simulatorDisplay.Cursor = System.Windows.Forms.Cursors.Cross;

			// disable other selectors
			DisableRoundSelector();
			DisableSquareSelector();

			selectorCurrentPoint = Point.Empty;

			// clear points
			polygonSelectorPoints.Clear();

			selectorType = SelectorType.Polygon;

			simulatorDisplay.MouseDown += polygonSelectorMouseDown;
			this.KeyDown += polygonSelectorKeyDown;

			selectorEnabled = true;
		}


		/// <summary>
		/// Disables the Polygon selector.
		/// </summary>
		private void DisablePolygonSelector()
		{
			selectorEnabled = false;
			simulatorDisplay.MouseDown -= polygonSelectorMouseDown;
			this.KeyDown -= polygonSelectorKeyDown;

			simulatorDisplay.Cursor = System.Windows.Forms.Cursors.Default;
		}


		/// <summary>
		/// Handles the MouseDownEvent event of the PolygonSelector control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="e">The <see cref="T:System.Windows.Forms.MouseEventArgs"/> instance containing the event data.</param>
		void PolygonSelector_MouseDownEvent(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			if (e.Button == MouseButtons.Left)
				polygonSelectorPoints.Add(TransformCoordinatePageToWorld(e.Location));

			if (e.Button == MouseButtons.Right)
			{
				DisablePolygonSelector();


				lock (displaySyncRoot)
				{
					// delete computers
					List<int> indices = new List<int>();

					for (int i = 0; i < completeComputerDataList.Count; i++)
					{
						if (PointInPoligon(completeComputerDataList[i].Location, polygonSelectorPoints))
							indices.Add(i);
					}

					// disable the computers
					Program.simulator.DisableComputers(indices, true);

					// refresh the lists
					if (Program.EnableRemoting)
						completeComputerDataList = Program.simulator.ComputerList;


				}
			}

		}


		/// <summary>
		/// Handles the KeyDownEvent event of the PolygonSelector control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="e">The <see cref="T:System.Windows.Forms.KeyEventArgs"/> instance containing the event data.</param>
		private void PolygonSelector_KeyDownEvent(object sender, KeyEventArgs e)
		{
			if (e.KeyCode == Keys.Escape)
			{
				DisablePolygonSelector();
				selectorEnabled = false;
				e.Handled = true;
			}
		}


		/// <summary>
		/// Check if point is in polygon
		/// </summary>
		/// <param name="p">The p.</param>
		/// <param name="poly">The poly.</param>
		/// <returns></returns>
		private bool PointInPoligon(Point p, List<Point> poly)
		{
			// the crossing number counter
			int cn = 0;
			// loop through all edges of the polygon
			for (int i = 0; i < poly.Count; i++)
			{
				// edge from V[i] to V[i+1]
				if (((poly[i].Y <= p.Y) && (poly[(i + 1) % poly.Count].Y > p.Y)) // an upward crossing
					|| ((poly[i].Y > p.Y) && (poly[(i + 1) % poly.Count].Y <= p.Y)))
				{
					// a downward crossing
					// compute the actual edge-ray intersect x-coordinate
					float vt = (float)(p.Y - poly[i].Y) / (poly[(i + 1) % poly.Count].Y - poly[i].Y);
					if (p.X < poly[i].X + vt * (poly[(i + 1) % poly.Count].X - poly[i].X)) // P.x < intersect
						++cn; // a valid crossing of y=P.y right of P.x
				}
			}
			return (cn % 2 == 1); // 0 if even (out), and 1 if odd (in)
		}

		#endregion


	}
}
