/*
	Paintable Computing Simulation

	License

	This software (Paintable Computing Simulation and all parts of its 
	source code) is provided "as is", without any warranty and it is 
	the property of its respective authors. Use of this software is 
	free to everyone, except in commercial and/or military purposes 
	without written permission of its authors. Disassembling the 
	executable code of this software is forbidden for any purpose 
	without written permission of its authors.

	If you intend to use any part of the source code of this software 
	or to develop this software any further, you must include this 
	license in every source file and every About box, without changing 
	it; only appending the license is allowed. And if you use any part 
	of the source code of this software, or develop it further, please 
	send an e-mail to its authors.

	Authors of this software are not responsible for any consequences 
	which use of this software, use of any part of its source or its 
	further development may cause.

	All software and parts of source code used in the development of 
	this software are property of their respective owners.

	Good luck,
	Authors

	Authors (alphabetically):
	Fabijan Lukin
		fabijan@gmail.com
		fabijan.lukin@fer.hr
	Fran Pregernik
		fran.pregernik@gmail.com
		fran.pregernik@fer.hr
	Tomislav Sukser
		tomislav.sukser@gmail.com
		tomislav.sukser@zg.t-com.hr
		tomislav.sukser@fer.hr


	Note:
	Paintable computing was firstly introduced in Programming a 
	Paintable Computer written by William Joseph Butera.


    Copyright (c) 2005,2006. All rights reserved.
    4-6/2006

 */
using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using System.Drawing;
using PaintableSimulator.Engine;
using PaintableSimulator.Engine.PaintableComputers.OperatingSystem;
using System.Runtime.Remoting;
using System.Net.Sockets;

namespace PaintableSimulator.SmartClient
{
	public partial class SimulatorClient
	{
		/// <summary>
		/// 
		/// </summary>
		private MouseEventHandler debugDisplayMouseMove;

		/// <summary>
		/// 
		/// </summary>
		private MouseEventHandler debugDisplayMouseClick;

		/// <summary>
		/// 
		/// </summary>
		private List<DebugVisualizer> debugVisualisers = new List<DebugVisualizer>();

		private SimpleComputerInfo temoraryDebugComputer;

		/// <summary>
		/// Initializes the debugger.
		/// </summary>
		private void InitializeDebugger()
		{
			debugDisplayMouseMove = new MouseEventHandler(debugger_MouseMove);
			debugDisplayMouseClick = new MouseEventHandler(debugger_MouseClick);
		}

		/// <summary>
		/// Disables the debugger.
		/// </summary>
		private void DisableDebugger()
		{
			temoraryDebugComputer = null;

			// close all debuggers
			for (int i = debugVisualisers.Count - 1; i >= 0; i--)
				debugVisualisers[i].Close();


			try
			{
				DisableDebugSelector();
			}
			catch
			{ }
		}

		/// <summary>
		/// Enables the computer selector.
		/// </summary>
		private void EnableDebugSelector()
		{
			simulatorDisplay.Cursor = Cursors.Cross;
			simulatorDisplay.MouseMove += debugDisplayMouseMove;
			simulatorDisplay.MouseClick += debugDisplayMouseClick;
		}

		/// <summary>
		/// Disable the computer selector.
		/// </summary>
		private void DisableDebugSelector()
		{
			simulatorDisplay.Cursor = Cursors.Default;
			simulatorDisplay.MouseMove -= debugDisplayMouseMove;
			simulatorDisplay.MouseClick -= debugDisplayMouseClick;
		}

		/// <summary>
		/// Draws the debug data.
		/// </summary>
		/// <param name="graphics">The graphics.</param>
		void DrawDebugData(Graphics graphics)
		{
			for (int j = 0; j < debugVisualisers.Count; j++)
			{
				SimpleComputerInfo debuggedComputer = completeComputerDataList[debugVisualisers[j].ComputerIndex];

				// draw wireless range
				uint radius = Program.simulator.SimulationSettings.WirelessRadius;
				Point center = new Point((int)(debuggedComputer.Location.X - radius), (int)(debuggedComputer.Location.Y - radius));
				SolidBrush fill = new SolidBrush(Color.FromArgb(100, 255, 255, 0));
				graphics.FillEllipse(fill, CreateCircleAroundPoint(debuggedComputer.Location, radius));
				graphics.DrawEllipse(Pens.Yellow, CreateCircleAroundPoint(debuggedComputer.Location, radius));

				// colorize  selected computer
				Rectangle pc = CreateCircleAroundPoint(debuggedComputer.Location, 3);
				graphics.FillEllipse(Brushes.Yellow, pc);


				// colorize neighbours
				for (int i = 0; i < debuggedComputer.NeighboursIndices.Count; i++)
				{
					SimpleComputerInfo selected = completeComputerDataList[debuggedComputer.NeighboursIndices[i]];
					Rectangle innerCircle = CreateCircleAroundPoint(selected.Location, 3);
					Color color = Color.FromArgb(100, Color.Red);
					graphics.FillEllipse(new SolidBrush(color), innerCircle);

				}

			}

			// Draw selection
			if (temoraryDebugComputer != null)
			{

				// draw wireless range
				uint radius = Program.simulator.SimulationSettings.WirelessRadius;
				Point center = new Point((int)(temoraryDebugComputer.Location.X - radius), (int)(temoraryDebugComputer.Location.Y - radius));
				SolidBrush fill = new SolidBrush(Color.FromArgb(100, 255, 255, 0));
				graphics.FillEllipse(fill, CreateCircleAroundPoint(temoraryDebugComputer.Location, radius));
				graphics.DrawEllipse(Pens.Yellow, CreateCircleAroundPoint(temoraryDebugComputer.Location, radius));

				// colorize  selected computer
				Rectangle pc = CreateCircleAroundPoint(temoraryDebugComputer.Location, 2);
				graphics.FillEllipse(Brushes.Yellow, pc);


				// colorize neighbours
				for (int i = 0; i < temoraryDebugComputer.NeighboursIndices.Count; i++)
				{
					SimpleComputerInfo selected = completeComputerDataList[temoraryDebugComputer.NeighboursIndices[i]];
					Rectangle innerCircle = CreateCircleAroundPoint(selected.Location, 3);
					Color color = Color.FromArgb(100, Color.Red);
					graphics.FillEllipse(new SolidBrush(color), innerCircle);
				}

			}
		}

		/// <summary>
		/// Determines whether [is point within area] [the specified center].
		/// </summary>
		/// <param name="center">The center.</param>
		/// <param name="selected">The selected.</param>
		/// <returns>
		/// 	<c>true</c> if [is point within area] [the specified center]; otherwise, <c>false</c>.
		/// </returns>
		private bool IsPointWithinArea(Point center, Point selected)
		{
			int dx = (center.X - selected.X);
			int dy = (center.Y - selected.Y);
			double distance = Math.Sqrt(dx * dx + dy * dy);
			return distance < 4.0;
		}


		/// <summary>
		/// Handles the MouseMove event of the display during debugging control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="e">The <see cref="T:System.Windows.Forms.MouseEventArgs"/> instance containing the event data.</param>
		private void debugger_MouseMove(object sender, MouseEventArgs e)
		{
			Point pt = TransformCoordinatePageToWorld(e.Location);

			for (int i = 0; i < completeComputerDataList.Count; i++)
				if (IsPointWithinArea(completeComputerDataList[i].Location, pt))
					temoraryDebugComputer = completeComputerDataList[i];

			//redraw if paused
			if (IsSimulatorPaused())
				simulatorDisplay.Invalidate();
		}


		/// <summary>
		/// Handles the MouseClick event of the display during debugging.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="e">The <see cref="T:System.Windows.Forms.MouseEventArgs"/> instance containing the event data.</param>
		private void debugger_MouseClick(object sender, MouseEventArgs e)
		{
			if (e.Button == MouseButtons.Left)
			{
				if (temoraryDebugComputer != null)
				{
					DisableDebugSelector();

					lock (debugSyncRoot)
					{

						// add new debug visualizer
						int index = completeComputerDataList.IndexOf(temoraryDebugComputer);
						debugVisualisers.Add(new DebugVisualizer(index,temoraryDebugComputer,Program.simulator.GetComputer(index), this));
						debugVisualisers[debugVisualisers.Count - 1].Show();
						debugVisualisers[debugVisualisers.Count - 1].RefreshDebugInfo();
					}

					stopDebugger.Enabled = true;
					stopDebuggerToolStripMenuItem.Enabled = true;

				}
			}


			selectComputerToDebug.Enabled = true;
			selectComputerToolStripMenuItem.Enabled = true;

			stopDebugger.Enabled = true;
			stopDebuggerToolStripMenuItem.Enabled = true;

			//redraw if paused
			if (IsSimulatorPaused())
				simulatorDisplay.Invalidate();

			temoraryDebugComputer = null;
		}


		/// <summary>
		/// Removes the debug visualizer.
		/// </summary>
		/// <param name="dv">The dv.</param>
		internal void RemoveDebugVisualizer(DebugVisualizer dv)
		{
			lock (debugSyncRoot)
			{
				debugVisualisers.Remove(dv);

				if (debugVisualisers.Count == 0)
				{
					stopDebugger.Enabled = false;
					stopDebuggerToolStripMenuItem.Enabled = false;
				}
			}
		}



		/// <summary>
		/// Handles the Click event of the selectComputerToDebug control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="e">The <see cref="T:System.EventArgs"/> instance containing the event data.</param>
		private void selectComputerToDebug_Click(object sender, EventArgs e)
		{
			if (IsSimulatorDataAvailable())
			{
				EnableDebugSelector();
			}

			selectComputerToDebug.Enabled = false;
			selectComputerToolStripMenuItem.Enabled = false;

		}


		/// <summary>
		/// Handles the Click event of the stopDebugger control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="e">The <see cref="T:System.EventArgs"/> instance containing the event data.</param>
		private void stopDebugger_Click(object sender, EventArgs e)
		{
			DisableDebugger();
		}

	}
}
