/*
	Paintable Computing Simulation

	License

	This software (Paintable Computing Simulation and all parts of its 
	source code) is provided "as is", without any warranty and it is 
	the property of its respective authors. Use of this software is 
	free to everyone, except in commercial and/or military purposes 
	without written permission of its authors. Disassembling the 
	executable code of this software is forbidden for any purpose 
	without written permission of its authors.

	If you intend to use any part of the source code of this software 
	or to develop this software any further, you must include this 
	license in every source file and every About box, without changing 
	it; only appending the license is allowed. And if you use any part 
	of the source code of this software, or develop it further, please 
	send an e-mail to its authors.

	Authors of this software are not responsible for any consequences 
	which use of this software, use of any part of its source or its 
	further development may cause.

	All software and parts of source code used in the development of 
	this software are property of their respective owners.

	Good luck,
	Authors

	Authors (alphabetically):
	Fabijan Lukin
		fabijan@gmail.com
		fabijan.lukin@fer.hr
	Fran Pregernik
		fran.pregernik@gmail.com
		fran.pregernik@fer.hr
	Tomislav Sukser
		tomislav.sukser@gmail.com
		tomislav.sukser@zg.t-com.hr
		tomislav.sukser@fer.hr


	Note:
	Paintable computing was firstly introduced in Programming a 
	Paintable Computer written by William Joseph Butera.


    Copyright (c) 2005,2006. All rights reserved.
    4-6/2006

 */
using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using PaintableSimulator.Engine.PaintableComputers.OperatingSystem;
using System.Reflection;
using System.ComponentModel;
using System.Collections.ObjectModel;
using PaintableSimulator.Engine;
using System.Drawing.Drawing2D;
using System.Drawing;
using System.IO;
using System.Runtime.Serialization.Formatters.Binary;
using PaintableSimulator.SmartClient.Resources;

namespace PaintableSimulator.SmartClient
{
	public partial class SimulatorClient
	{

		private MouseEventHandler processFragmentLoadMouseClickHandler;
		private MouseEventHandler processFragmentLoadMouseMoveHandler;

		private SimpleComputerInfo highlightedComputer;

		/// <summary>
		/// Reads the ProcessFragment subdirectory for ProcessFragment dll's
		/// </summary>
		private void ScanProcesFragments(TreeView tw)
		{

			// clear the treeview
			tw.Nodes.Clear();

			List<FriendlyType> fragments = null;

			try
			{
				fragments = Program.simulator.Plugins.ValidProcessFragments;
			}
			catch (System.Runtime.Remoting.RemotingException ex)
			{
				Program.HandleIrrecoverableRemotingException(ex);
				return;
			}
			catch (System.Net.Sockets.SocketException ex)
			{
				Program.HandleIrrecoverableRemotingException(ex);
				return;
			}
			catch(Exception ex)
			{
				if (Program.log.IsErrorEnabled)
					Program.log.ErrorFormat(UserMessages.ErrorIn, "SimulatorClientScanProcesFragments(TreeView)", ex);
				return;
			}

			if (fragments == null)
				return;

			// load assemblies
			for (int i = 0; i < fragments.Count; i++)
			{
				string assemblyName = fragments[i].InnerType.Assembly.GetName().Name;

				TreeNode assemblyNode = null;
				if (tw.Nodes.ContainsKey(fragments[i].InnerType.Assembly.FullName))
					assemblyNode = tw.Nodes[fragments[i].InnerType.Assembly.FullName];
				else
					assemblyNode = tw.Nodes.Add(fragments[i].InnerType.Assembly.FullName, assemblyName);

				// make an entry for each
				TreeNode typeNode = new TreeNode(fragments[i].UserDefinedName);

				// The tag contains the type that is used to instantiate the pfrag
				typeNode.Tag = fragments[i];

				// set tool tip
				typeNode.ToolTipText = fragments[i].UserDefinedDescription;

				// add it to the assembly node
				assemblyNode.Nodes.Add(typeNode);
				
			}


			// expand all nodes
			tw.ExpandAll();
		}


		/// <summary>
		/// Initializes the process fragment list.
		/// </summary>
		public void InitializeProcessFragments()
		{
			processFragmentLoadMouseClickHandler = new MouseEventHandler(loadProcessFragment_MouseClick);
			processFragmentLoadMouseMoveHandler = new MouseEventHandler(loadProcessFragment_MouseMove);
		}


		/// <summary>
		/// Handles the AfterSelect event of the processFragmentTree control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="e">The <see cref="T:System.Windows.Forms.TreeViewEventArgs"/> instance containing the event data.</param>
		private void processFragmentTree_AfterSelect(object sender, TreeViewEventArgs e)
		{
			
			if (processFragmentTree.SelectedNode.Tag != null)
			{
				FriendlyType pfrag = (FriendlyType)processFragmentTree.SelectedNode.Tag;

				//enable controls
				processFragmentInitialization.Enabled = true;
				processFragmentInitialization.Visible = true;

				// determine if it is neccesary to load the settings form
				bool showForm = false;
				PropertyInfo[] properties = pfrag.InnerType.GetProperties();
				for (int i = 0; i < properties.Length; i++)
					if (properties[i].GetCustomAttributes(typeof(BrowsableAttribute), false).Length != 0)
						showForm = true;

				try
				{
					processFragmentProperties.SelectedObject = Activator.CreateInstance(pfrag);
				}
				catch (Exception ex)
				{
					MessageBox.Show(ex.ToString());
					return;
				}

				// show this form (if neccesary and load it's grid with ProcessFragmentState's properties
				if (showForm)
					processFragmentProperties.Enabled = true;
				else
					processFragmentProperties.Enabled = false;

				loadProcessFragment.Enabled = true;

			}
			else
			{
				//disable initialzation controls
				processFragmentInitialization.Enabled = false;
				loadProcessFragment.Enabled = false;
				processFragmentInitialization.Visible = false;
				
			}
		}



		/// <summary>
		/// Handles the Click event of the loadProcessFragment control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="e">The <see cref="T:System.EventArgs"/> instance containing the event data.</param>
		private void loadProcessFragment_Click(object sender, EventArgs e)
		{
			if (!IsSimulatorDataAvailable())
			{
				MessageBox.Show(UserMessages.SimulatorNotStartedOrUnavailable);
				return;
			}

			if (processFragmentTree.SelectedNode.Tag == null)
			{
				MessageBox.Show(UserMessages.NoPfragChosen);
				return;
			}

			processFragmentProperties.Enabled = false;
			loadProcessFragment.Enabled = false;

			// Program.simulator is working, enable display clicking
			simulatorDisplay.MouseClick += processFragmentLoadMouseClickHandler;
			simulatorDisplay.MouseMove += processFragmentLoadMouseMoveHandler;
			simulatorDisplay.Cursor = Cursors.Cross;
		}


		/// <summary>
		/// Handles the MouseMove event of the loadProcessFragment control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="e">The <see cref="T:System.Windows.Forms.MouseEventArgs"/> instance containing the event data.</param>
		private void loadProcessFragment_MouseMove(object sender, MouseEventArgs e)
		{
			// ToDo: implement Paintable computer highlight
			Point pt = TransformCoordinatePageToWorld(e.Location);

			for (int i = 0; i < completeComputerDataList.Count; i++)
				if (IsPointWithinArea(completeComputerDataList[i].Location, pt))
					highlightedComputer = completeComputerDataList[i];

			//redraw if paused
			if (IsSimulatorPaused())
				simulatorDisplay.Invalidate();
		}


		/// <summary>
		/// Draws the debug data.
		/// </summary>
		/// <param name="graphics">The graphics.</param>
		void DrawHighlightedComputer(Graphics graphics)
		{
			if (highlightedComputer == null)
				return;


			// colorize  selected computer
			Rectangle outerCircle = CreateCircleAroundPoint(highlightedComputer.Location, 7);
			Rectangle innerCircle = CreateCircleAroundPoint(highlightedComputer.Location, 4);
			graphics.FillEllipse(Brushes.Yellow, outerCircle);
			graphics.FillEllipse(Brushes.White, innerCircle);



		}
		

		/// <summary>
		/// 
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="e">The <see cref="T:System.MouseEventArgs"/> instance containing the event data.</param>
		private void loadProcessFragment_MouseClick(object sender, MouseEventArgs e)
		{
			simulatorDisplay.Cursor = Cursors.Default;
			simulatorDisplay.MouseClick -= processFragmentLoadMouseClickHandler;
			simulatorDisplay.MouseMove -= processFragmentLoadMouseMoveHandler;

			// right mouse click loads the process fragment
			if (e.Button == MouseButtons.Left)
			{
				Point pt = TransformCoordinatePageToWorld(e.Location);

				for (int i = 0; i < completeComputerDataList.Count; i++)
				{
					int dx = pt.X - completeComputerDataList[i].Location.X;
					int dy = pt.Y - completeComputerDataList[i].Location.Y;

					int distance = (int)Math.Sqrt(dx * dx + dy * dy);

					if ((distance < 4))
					{
						// load the process fragment
						if (processFragmentTree.SelectedNode.Tag == null)
							return;

						FriendlyType pfrag = (FriendlyType)processFragmentTree.SelectedNode.Tag;

						try
						{
							Program.simulator.InvadeComputer(i, (ProcessFragmentBase)processFragmentProperties.SelectedObject);
							completeComputerDataList[i].IsPortal = true;
						}
						catch (Exception ex)
						{
							MessageBox.Show(ex.ToString());
						}

						// select the loaded process fragment to colorize homepages
						if (!lockColorizer.Checked)
						{
							FriendlyType defaultFragmentColorizer = pfrag.DefaultColorizer;
							homepageColorizers.SelectedItem = defaultFragmentColorizer;
						}

						// disable highlighting a computer
						highlightedComputer = null;

						// hide fragment settings
						processFragmentProperties.Enabled = false;
						processFragmentProperties.SelectedObject = null;
						loadProcessFragment.Enabled = false;
						processFragmentInitialization.Visible = false;
						processFragmentTree.SelectedNode = null;

						//redraw if paused
						if (IsSimulatorPaused())
							simulatorDisplay.Invalidate();

						return;

					}

				}

			}
			else if (e.Button == MouseButtons.Right)
			{
				// disable highlighting a computer
				highlightedComputer = null;
			}


			// enable fragment settings
			processFragmentProperties.Enabled = true;
			loadProcessFragment.Enabled = true;







		
		}
	}
}
