/*
	Paintable Computing Simulation

	License

	This software (Paintable Computing Simulation and all parts of its 
	source code) is provided "as is", without any warranty and it is 
	the property of its respective authors. Use of this software is 
	free to everyone, except in commercial and/or military purposes 
	without written permission of its authors. Disassembling the 
	executable code of this software is forbidden for any purpose 
	without written permission of its authors.

	If you intend to use any part of the source code of this software 
	or to develop this software any further, you must include this 
	license in every source file and every About box, without changing 
	it; only appending the license is allowed. And if you use any part 
	of the source code of this software, or develop it further, please 
	send an e-mail to its authors.

	Authors of this software are not responsible for any consequences 
	which use of this software, use of any part of its source or its 
	further development may cause.

	All software and parts of source code used in the development of 
	this software are property of their respective owners.

	Good luck,
	Authors

	Authors (alphabetically):
	Fabijan Lukin
		fabijan@gmail.com
		fabijan.lukin@fer.hr
	Fran Pregernik
		fran.pregernik@gmail.com
		fran.pregernik@fer.hr
	Tomislav Sukser
		tomislav.sukser@gmail.com
		tomislav.sukser@zg.t-com.hr
		tomislav.sukser@fer.hr


	Note:
	Paintable computing was firstly introduced in Programming a 
	Paintable Computer written by William Joseph Butera.


    Copyright (c) 2005,2006. All rights reserved.
    4-6/2006

 */

using System;
using System.Collections.Generic;
using System.Text;
using System.IO;

using log4net.Util;
using log4net.Layout;
using log4net.Core;
using System.Windows.Forms;
using log4net.Appender;

namespace PaintableSimulator.SmartClient.Logging
{


		/// <summary>
		/// 
		/// </summary>
		internal class TextBoxAppender : AppenderSkeleton
		{
			private static TextBox outputTextBox;

			private static Queue<String> logQueue = new Queue<string>();


			/// <summary>
			/// 
			/// </summary>
			public static TextBox OutputTextBox
			{
				get { return TextBoxAppender.outputTextBox; }
				set { TextBoxAppender.outputTextBox = value; }
			}

			private delegate void AppendLogHandler();


			/// <summary>
			/// Appends the log.
			/// </summary>
			internal static void AppendLog()
			{
				while (logQueue.Count != 0)
				{
					outputTextBox.Text = logQueue.Dequeue() + outputTextBox.Text;
				}
			}


			#region Public Instance Constructors

			/// <summary>
			/// Initializes a new instance of the <see cref="TextBoxAppender" /> class.
			/// </summary>
			public TextBoxAppender()
			{
			}

			#endregion

			#region Override implementation of AppenderSkeleton

			/// <summary>
			/// This method determines if there is a sense in attempting to append.
			/// </summary>
			/// <remarks>
			/// <para>
			/// This method checked if an output target has been set and if a
			/// layout has been set. 
			/// </para>
			/// </remarks>
			/// <returns><c>false</c> if any of the preconditions fail.</returns>
			override protected bool PreAppendCheck()
			{
				if (!base.PreAppendCheck())
				{
					return false;
				}

				return true;
			}

			/// <summary>
			/// This method is called by the <see cref="AppenderSkeleton.DoAppend"/>
			/// method. 
			/// </summary>
			/// <param name="loggingEvent">The event to log.</param>
			/// <remarks>
			/// <para>
			/// Writes a log statement to the output stream if the output stream exists 
			/// and is writable.  
			/// </para>
			/// <para>
			/// The format of the output will depend on the appender's layout.
			/// </para>
			/// </remarks>
			override protected void Append(LoggingEvent loggingEvent)
			{
				if (outputTextBox != null)
				{
					try
					{
						string message = String.Format("{0} {1}\r\n", loggingEvent.TimeStamp.ToShortTimeString(), loggingEvent.RenderedMessage);
						logQueue.Enqueue(message);
						outputTextBox.BeginInvoke(new AppendLogHandler(TextBoxAppender.AppendLog));
					}
					catch { }
				}
			}

			#endregion Override implementation of AppenderSkeleton


		}
	}
